###############################################################################
### Purpose: rssd.VSA.NR5G_K144 test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.NR5G_K144     import VSA

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_5GNR()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_5GNR_Direction(self):
        self.FSW.Set_5GNR_Direction('UL')
        getVal = self.FSW.Get_5GNR_Direction()
        if self.FSW.connected: self.assertEqual(getVal,'UL')
        self.FSW.Set_5GNR_Direction('DL')
        getVal = self.FSW.Get_5GNR_Direction()
        if self.FSW.connected: self.assertEqual(getVal,'DL')

    def test_FSW_5GNR_Ex_Meas_Multi_CC(self):
        self.FSW.Init_5GNR_Meas('EVM')
        # Configure setup
        self.FSW.Set_5GNR_CellID(1)
        self.FSW.Set_5GNR_SubFrameCount(2)
        self.FSW.Set_5GNR_Result_View('ALL')        # Multi CC results
        self.FSW.Set_5GNR_EVMUnit('DB')
        self.FSW.Get_5GNR_Params_EVM()

    def test_FSW_5GNR_Ex_SEM(self):
        self.FSW.Init_5GNR_SEM()
        self.FSW.Set_5GNR_SEM_Freq(20e9)
        self.FSW.Set_5GNR_SEM_SubBlockNum(1)
        self.FSW.Get_5GNR_SEM()

    def test_FSW_5GNR_FreqRange(self):
        self.FSW.Set_5GNR_Direction('UL')
        self.FSW.Set_5GNR_FreqRange('LOW')
        getVal = self.FSW.Get_5GNR_FreqRange()
        if self.FSW.connected: self.assertEqual(getVal,'LOW')

        self.FSW.Set_5GNR_FreqRange('MIDD')
        getVal = self.FSW.Get_5GNR_FreqRange()
        if self.FSW.connected: self.assertEqual(getVal,'MIDD')

        self.FSW.Set_5GNR_FreqRange('HIGH')
        getVal = self.FSW.Get_5GNR_FreqRange()
        if self.FSW.connected: self.assertEqual(getVal,'HIGH')

    def test_FSW_5GNR_Get_DL(self):
        self.FSW.Set_5GNR_Direction('DL')
        nullVal = self.FSW.Get_5GNR_CC_Freq()
        nullVal = self.FSW.Get_5GNR_Direction()
        nullVal = self.FSW.Get_5GNR_FreqRange()
        nullVal = self.FSW.Get_5GNR_RefA()
        nullVal = self.FSW.Get_5GNR_ChannelBW()
        nullVal = self.FSW.Get_5GNR_TransPrecoding()
        nullVal = self.FSW.Get_5GNR_PhaseCompensate()
        nullVal = self.FSW.Get_5GNR_SSB_SubSpace()
        nullVal = self.FSW.Get_5GNR_BWP_SubSpace()
        nullVal = self.FSW.Get_5GNR_BWP_Count()
        nullVal = self.FSW.Get_5GNR_BWP_ResBlock()
        nullVal = self.FSW.Get_5GNR_BWP_ResBlockOffset()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_Modulation()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_ResBlock()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_ResBlockOffset()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_SymbNum()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_SymbOff()
        if self.FSW.connected: nullVal = self.FSW.Get_5GNR_BWP_Center()
        ### "=DMRS="
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_Config()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_Mapping()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_1stDMRSSym()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_AddPosition()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_MSymbLen()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_SeqGenSeed()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_RelPwr()
        ### "=PTRS=")
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_State()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_L()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_K()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_Pow()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_RE_Offset()

    def test_FSW_5GNR_Get_PTRS(self):
        self.FSW.Set_5GNR_Direction('DL')
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_State()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_L()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_K()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_Pow()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_RE_Offset()

    def test_FSW_5GNR_Get_Misc(self):
        self.FSW.Get_5GNR_ACLR()
        self.FSW.Get_5GNR_BWP_SlotNum()
        self.FSW.Get_5GNR_CC_Offset()
        self.FSW.Get_5GNR_Meas_ACLR()

    def test_FSW_5GNR_Get_UL(self):
        self.FSW.Set_5GNR_Direction('UL')
        nullVal = self.FSW.Get_5GNR_CC_Freq()
        nullVal = self.FSW.Get_5GNR_Direction()
        nullVal = self.FSW.Get_5GNR_FreqRange()
        nullVal = self.FSW.Get_5GNR_RefA()
        nullVal = self.FSW.Get_5GNR_ChannelBW()
        nullVal = self.FSW.Get_5GNR_TransPrecoding()
        nullVal = self.FSW.Get_5GNR_PhaseCompensate()
        nullVal = self.FSW.Get_5GNR_SSB_SubSpace()
        ### "=User="
        nullVal = self.FSW.Get_5GNR_BWP_SubSpace()
        nullVal = self.FSW.Get_5GNR_BWP_Count()
        nullVal = self.FSW.Get_5GNR_BWP_ResBlock()
        nullVal = self.FSW.Get_5GNR_BWP_ResBlockOffset()
        ### "==Ch=="
        nullVal = self.FSW.Get_5GNR_BWP_Ch_Modulation()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_ResBlock()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_ResBlockOffset()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_SymbNum()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_SymbOff()
        if self.FSW.connected: nullVal = self.FSW.Get_5GNR_BWP_Center()
        ### "=DMRS="
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_Config()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_Mapping()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_1stDMRSSym()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_AddPosition()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_MSymbLen()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_SeqGenMeth()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_SeqGenSeed()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_DMRS_RelPwr()
        ### "=PTRS=")
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_State()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_L()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_K()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_Pow()
        nullVal = self.FSW.Get_5GNR_BWP_Ch_PTRS_RE_Offset()

    def test_FSW_5GNR_InstrState(self):
        self.FSW.Get_5GNR_TM_Cat()
        self.FSW.Set_5GNR_TM('NR-FR1-TM1_1__FDD_15MHz_30kHz')
        self.FSW.Set_5GNR_savesetting('test')
        self.FSW.Set_5GNR_AllocFile('test')         #mmm
        self.FSW.Set_5GNR_AllocFileSave('test')     #mmm

    def test_FSW_5GNR_Set_DL(self):
        self.FSW.Set_5GNR_Direction('DL')
        self.FSW.Set_5GNR_CC_Num(1)
        self.FSW.Set_5GNR_TransPrecoding('OFF')
        self.FSW.Set_5GNR_PhaseCompensate('ON')
        self.FSW.Set_5GNR_PhaseCompensate_Freq(1e6)
        self.FSW.Set_5GNR_FreqRange('HIGH')
        self.FSW.Set_5GNR_ChannelBW(100)
        self.FSW.Set_5GNR_BWP_SubSpace(120)
        self.FSW.Set_5GNR_BWP_ResBlock(66)
        self.FSW.Set_5GNR_BWP_ResBlockOffset(0)
        self.FSW.Set_5GNR_BWP_Ch_ResBlock(66)
        self.FSW.Set_5GNR_BWP_Corset_ResBlock(66)
        self.FSW.Set_5GNR_BWP_Ch_ResBlockOffset(0)
        self.FSW.Set_5GNR_BWP_Ch_Modulation('QPSK')
        # self.FSW.Set_5GNR_SSB()

    def test_FSW_5GNR_Set_DMRS(self):
        self.FSW.Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(2)
        self.FSW.Set_5GNR_BWP_Ch_DMRS_AddPosition(0)
        self.FSW.Set_5GNR_BWP_Ch_DMRS_Config(1)
        self.FSW.Set_5GNR_BWP_Ch_DMRS_Mapping('A')
        self.FSW.Set_5GNR_BWP_Ch_DMRS_MSymbLen(1)
        self.FSW.Set_5GNR_BWP_Ch_DMRS_RelPwr(0)
        self.FSW.Set_5GNR_BWP_Ch_DMRS_SeqGenMeth('NICD')
        self.FSW.Set_5GNR_BWP_Ch_DMRS_SeqGenSeed(0)

    def test_FSW_5GNR_Set_UL(self):
        self.FSW.Set_5GNR_Direction('UL')
        self.FSW.Set_5GNR_CC_Num(1)
        self.FSW.Set_5GNR_TransPrecoding('OFF')
        self.FSW.Set_5GNR_PhaseCompensate('OFF')
        self.FSW.Set_5GNR_FreqRange('HIGH')
        self.FSW.Set_5GNR_ChannelBW(100)
        self.FSW.Set_5GNR_BWP_SubSpace(120)
        self.FSW.Set_5GNR_BWP_ResBlock(66)
        self.FSW.Set_5GNR_BWP_ResBlockOffset(0)
        self.FSW.Set_5GNR_BWP_Ch_ResBlock(66)
        self.FSW.Set_5GNR_BWP_Corset_ResBlock(66)
        self.FSW.Set_5GNR_BWP_Ch_ResBlockOffset(0)
        self.FSW.Set_5GNR_BWP_Ch_Modulation('QPSK')

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_VSA_5GNR
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""rssd.VSA.Common_IQ test
"""
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.Common        import VSA              #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_IQ()

    def tearDown(self):                                 #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_IQ_Common(self):
        self.FSW.Set_IQ_BW(100e6)
        self.FSW.Set_IQ_SamplingRate(100e6)
        self.FSW.Set_IQ_RecLength(100)
        self.FSW.Set_IQ_Samples(100)
        self.FSW.Set_IQ_Time(1e-3)
        getVal = self.FSW.Get_IQ_RecLength()
        getVal = self.FSW.Get_IQ_SamplingRate()

    def test_FSW_Get_IQData(self):
        self.FSW.Set_IQ_RecLength(100)
        self.FSW.Set_SweepCont(0)
        getVal = self.FSW.Get_IQ_Data()
        getVal = self.FSW.Get_IQ_Data_Ascii(10)         # Get 10IQ at a time
        # if self.FSW.connected == 1: getVal = self.FSW.Get_IQ_Data_Ascii(5)
        getVal = self.FSW.Get_IQ_Data_Ascii2()
        if self.FSW.connected == 1: getVal = self.FSW.Get_IQ_Data_Bin()

    def test_FSW_Set_IQ_ALCR(self):
        self.FSW.Set_IQ_ACLR(9e6,10e6)
        self.FSW.Get_Mkr_BandACLR()

    def test_FSW_Set_IQ_Adv(self):
        self.FSW.Set_IQ_Adv_Mode(0)
        self.FSW.Set_IQ_Adv_Mode(1)
        self.FSW.Set_IQ_Adv_TransAlgo("AVER")
        self.FSW.Set_IQ_Adv_WindowLenth(101)
        self.FSW.Set_IQ_Adv_FFTLenth(4096+1)
        self.FSW.Set_IQ_Adv_Window('P5')
        self.FSW.Set_IQ_WideBandMax(80e6)

    def test_FSW_Set_IQSpectrum(self):
        self.FSW.Set_IQ_SpectrumWindow()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    # coverage run -a -m unittest -b -v test_HW_VSA_Common_IQ
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



'''RSSD test bus class'''
#pylint: disable=R0201,W0613

from rssd.bus.bus import bus

class jaTest(bus):                          #pylint: disable=R0205
    """Instrument Common functions"""
    def __init__(self):
        self.EOL        = '\n'
        self.connected          = 0

    def close(self):
        pass

    def open(self, resourceID, param=None):
        return self

    def query(self, SCPIstr):
        return "<notRead>"

    def read_raw(self):
        return b'1234567890'

    def timeout(self, seconds):
        pass

    def write_raw(self, SCPIstr):
        pass

    def write(self, SCPIstr):
        pass

    def SCPI_error(self):
        return ['0', 'No Error']



###############################################################################
### Purpose: rssd.VSG.Common test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '10.0.0.7'                                       #Get local machine name
host = '192.168.1.115'

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSG.Common    import VSG

class TestGeneral(unittest.TestCase):
    def setUp(self):                                    #run before each test
        self.SMW = VSG().jav_OpenTest(host)

    def tearDown(self):                                 #Run after each test
        self.assertEqual(self.SMW.jav_Error()[0],'0')
        self.SMW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_SMW_ALC(self):
        setVal = 'ON'
        self.SMW.Set_ALC_RFDriveAmp(setVal)

    def test_SMW_Arb_Freq(self):
        setVal = 10e6
        self.SMW.Set_ArbClockFreq(setVal)
        getVal = self.SMW.Get_ArbClockFreq()
        if self.SMW.connected: self.assertEqual(setVal,getVal)

    def test_SMW_Arb_State(self):
        setVal = '/var/user/UCS2010/GSM.wv'
        self.SMW.Set_ArbWv(setVal)
        self.SMW.Set_ArbState(1)
        self.SMW.Set_ArbState(0)
        getVal = self.SMW.Get_ArbName()
        self.SMW.Get_ArbInfo()
        nulVal = self.SMW.Get_PowerInfo()
        if self.SMW.connected: self.assertTrue(getVal.find(setVal) > -1)

    def test_SMW_BB_State(self):
        self.SMW.Set_BBState(1)
        self.SMW.Set_BBState(0)

    def test_SMW_Connect(self):
        if self.SMW.connected: self.assertEqual(self.SMW.Make,"Rohde&Schwarz")

    def test_SMW_CrestFactor(self):
        getVal = self.SMW.Get_CrestFactor()

    def test_SMW_OS(self):
        self.SMW.Set_OS_Dir('UCS2010')
        getVal = self.SMW.Get_OS_Dir()
        getVal = self.SMW.Get_OS_FileList()

    def test_SMW_Init_Wideband(self):
        self.SMW.Init_Wideband()

    def test_SMW_IQMod(self):
        self.SMW.Set_IQMod(1)
        self.SMW.Set_IQMod(0)
        self.SMW.Set_IQMod('OFF')
        self.SMW.Set_IQMod('ON')

    def test_SMW_Freq(self):
        setVal = 2e6
        self.SMW.Set_Freq(setVal)
        getVal = self.SMW.Get_Freq()
        if self.SMW.connected: self.assertEqual(setVal,getVal)

    def test_SMW_ListMode(self):
        self.SMW.Set_RFState(1)
        self.SMW.Set_ListMode_File('testListMode.lsw')
        self.SMW.Set_ListMode_File('testListMode')
        self.SMW.Set_ListMode('LIST')
        self.SMW.Set_ListMode_TrigSource('SING')
        self.SMW.Set_ListMode_Dwell(0.01)
        self.SMW.Set_ListMode_RMode('LIVE')
        # self.SMW.Set_ListMode_TrigExecute()
        # self.SMW.Set_ListMode_TrigWait()
        getVal = self.SMW.Get_ListMode_IndexCurr()
        getVal = self.SMW.Get_ListMode_IndexStop()
        self.SMW.Set_ListMode('CW')

    def test_SMW_ListMode_TrigSource(self):
        self.SMW.Set_ListMode_TrigSource('SING')
        self.SMW.Set_ListMode_TrigSource('AUTO')
        self.SMW.Set_ListMode_TrigSource('STEP')
        self.SMW.Set_ListMode_TrigSource('ESTEP')
        self.SMW.Set_ListMode_TrigSource('ESING')

    def test_SMW_NRP(self):
        self.SMW.Set_NRP_Mode('USER')
        self.SMW.Set_NRP_Freq(100e6)
        self.SMW.Get_NRPPower()

    def test_SMW_Optimize(self):
        if self.SMW.connected: self.SMW.Set_OptimizeIQ()        # timeout setting
        if self.SMW.connected: self.SMW.Set_OptimizeLevel()     # timeout setting
        if self.SMW.connected: self.SMW.Set_OptimizeAll()       # timeout setting

    def test_SMW_PhaseDelta(self):
        setVal = -10
        self.SMW.Set_PhaseDelta(setVal)

    def test_SMW_Pwr(self):
        setVal = -10
        self.SMW.Set_RFPwr(setVal)
        getVal = self.SMW.Get_PowerRMS()
        self.assertEqual(self.SMW.jav_Error()[0],'0')
        if self.SMW.connected: self.assertEqual(setVal,getVal)
    
    def test_SMW_PM(self):
        self.SMW.Set_PM_Source('LF1')
        self.SMW.Set_PM_State('ON')

    def test_SMW_SysConfigAll(self):
        getVal = self.SMW.Get_SysC_All()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    # unittest.main()
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=1).run(suite)



###############################################################################
### Purpose: rssd.VSG.LTE_K55 test
###              _   ___        __  _____         _
###             | | | \ \      / / |_   _|__  ___| |_
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '10.0.0.7'                                       #Get local machine name
host = '192.168.1.114'

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSG.LTE_K55   import VSG
from rssd.test.yaVISA   import jaVISA_mock              #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                    #run before each test
        self.SMW = VSG().jav_OpenTest(host)

    def tearDown(self):                                 #Run after each test
        self.assertEqual(self.SMW.jav_Error()[0],'0')
        self.SMW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_SMW_LTE_Get_DL(self):
        setVal = 'ON'
        self.SMW.Set_LTE_BBState(0)
        self.SMW.Set_LTE_Direction('DL')
        nullVal = self.SMW.Get_LTE_Direction()
        nullVal = self.SMW.Get_LTE_Duplex()
        nullVal = self.SMW.Get_LTE_ChBW()
        nullVal = self.SMW.Get_LTE_ResBlock()           # Need to test DL
        nullVal = self.SMW.Get_LTE_ResBlockOffset()     # Need to test DL
        nullVal = self.SMW.Get_LTE_Modulation()         # Need to test DL

    def test_SMW_LTE_Get_UL(self):
        setVal = 'ON'
        self.SMW.Set_LTE_BBState(0)
        self.SMW.Set_LTE_Direction('UL')
        nullVal = self.SMW.Get_LTE_Direction()
        nullVal = self.SMW.Get_LTE_Duplex()
        nullVal = self.SMW.Get_LTE_ChBW()
        nullVal = self.SMW.Get_LTE_ResBlock()           # Need to test DL
        nullVal = self.SMW.Get_LTE_ResBlockOffset()     # Need to test DL
        nullVal = self.SMW.Get_LTE_Modulation()         # Need to test DL

    def test_SMW_LTE_CC(self):
        self.SMW.Set_LTE_CC(1)
        self.SMW.Get_LTE_CC()

    def test_SMW_LTE_Direction(self):
        self.SMW.Set_LTE_Direction('BAD')
        self.SMW.Set_LTE_Direction('UL')
        getVal = self.SMW.Get_LTE_Direction()
        if self.SMW.connected: self.assertEqual(getVal,'UL')
        self.SMW.Set_LTE_Direction('DL')
        getVal = self.SMW.Get_LTE_Direction()
        if self.SMW.connected: self.assertEqual(getVal,'DL')

    def test_SMW_LTE_Direction2(self):
        self.SMW.Set_LTE_Duplex('TDD')
        self.SMW.Set_LTE_Duplex('TDD')

    def test_SMW_LTE_Set_UL(self):
        self.SMW.Set_Freq(2e9)
        self.SMW.Set_LTE_BBState('OFF')                     # Baseband OFF
        self.SMW.Set_LTE_Direction('UL')
        self.SMW.Set_LTE_ChBW(20)
        self.SMW.Set_LTE_ResBlock(66)
        self.SMW.Set_LTE_ResBlockOffset(0)
        self.SMW.Set_LTE_Modulation('QPSK')
        # self.SMW.Set_LTE_BBState('ON')
        # self.SMW.Set_RFState('ON')                          # Turn RF Output on
        self.SMW.Set_RFPwr(-50)                             # Output Power
        # self.SMW.delay(0.5)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""Purpose: Yet(Just) Another VISA wrapper

Descrip: Wrapper for common VISA commands
         properties for: Make; Model; Version; IDN; last error
          logSCPI --> file for"""
# pylint: disable=E0611,E0401,E0202,'pointless-statement'

import time
import pyvisa
import rssd.FileIO
from rssd.test.yaVISA       import jaVISA_mock
from rssd.RSI.time          import timer

class jaVisa(object):
    """Rohde & Schwarz VISA Class"""
    def __init__(self):
        self.dataIDN    = ""        # Raw IDN String
        self.Make       = ""        # IDN Make
        self.Model      = ""        # IDN Model
        self.Device     = ""        # IDN Device
        self.Version    = ""        # IDN Version
        self.debug      = 1         # Print or not
        self.EOL        = '\r\n'
        self.f          = ''        # Log File Object
        self.dLastErr   = ''        # Last error
        self.VISA       = ''    # '@py' for pyvisa-py
        self.K2         = 'NoVISA'

    def delay(self,sec):
        """delay in Sec"""
        time.sleep(sec)

    def jav_Clear(self):
        """Clear VISA Errors"""
        self.K2.clear()

    def jav_Close(self):
        """Close K2 Session"""
        try:
            errList = self.jav_ClrErr()
            self.K2.close()
            return errList
        except:
            pass

    def jav_ClrErr(self):
        """Read all SYST:ERR messages"""
        ErrList = []
        try:      #Instrument supports SYST:ERR?
            while True:
                RdStr = self.query("SYST:ERR?").strip()
                ErrList.append(RdStr)
                RdStrSplit = RdStr.split(',')
                if RdStr == "<notRead>" : break                     #No readstring
                if RdStrSplit[0] == "0" : break                     #Read 0 error:R&S
                if RdStrSplit[0] == "+0": break                     #Read 0 error:Other
                self.dLastErr = RdStr
                if self.debug: print("jav_ClrErr: %s-->%s"%(self.Model,RdStr))
        except:  #Instrument does not support SYST:ERR?
            if self.debug: print("jav_ClrErr: %s-->SYST:ERR not Supported"%(self.Model))
        return ErrList

    def jav_Error(self):
        """Read SYST:ERR?"""
        RdStr = self.query("SYST:ERR?").strip().split(',')
        return RdStr

    def jav_fileout(self, fily, outstr):
        try:
            if fily != '':
                fily.write(outstr.strip())
        except:
            pass

    def jav_IDN(self):
        """query *IDN?  Assign data to properties"""
        self.dataIDN = "Temp"                                       #Temp for self.query
        self.dataIDN = self.query("*IDN?").strip()
        if self.dataIDN != "<notRead>":                             #Data Returned?
            IDNStr = self.dataIDN.split(',')
            try:
                self.Make       = IDNStr[0]
                self.Model      = IDNStr[1]
                self.Device     = IDNStr[2]
                self.Version    = IDNStr[3]
            except:
                pass
        else:
            self.dataIDN = ""                                       #Reset if not read
        return self.dataIDN

    def jav_logscpi(self):
        self.f = rssd.FileIO()                                      #pylint:disable=E1101
        DataFile = self.f.init("yaVISA")                            #pylint:disable=W0612

    def jav_OPC_Wait(self, InCMD):
        """ Wait based on *OPC """
        start_time = time.time()
        self.write("*ESE 1")                                        #Event Status Enable
        self.write("*SRE 32")                                       #ServiceReqEnable-Bit5:Std Event
        self.write(InCMD + ";*OPC")                                 #Initiate Read.  *OPC will trigger ESR
        #print ('    OPC Wait: ' +InCMD)
        read = 0
        while (read & 1) != 1:                                      #Loop until done
            try:
                read = self.queryInt("*ESR?")                       #Poll EventStatReg-Bit0:Op Complete  (STB?)
            except:
                if self.debug: print("jav_OPCWai:*ESR? Error")
            time.sleep(0.5)
            delta = (time.time() - start_time)
            if delta > 300:
                if self.debug: print("jav_OPCWai: timeout")
                break
        if self.debug: print('jav_OPCWai: %0.2fsec'%(delta))
        self.jav_ClrErr()
        return delta

    def jav_Open(self, IPAddr, fily=''):
        """Open VISA session"""
        #  VISA: 'TCPIP0::'+IP_Address+'::INSTR'
        #  VISA: 'TCPIP0::'+IP_Address+'::inst0'
        #  VISA: 'TCPIP0::'+IP_Address+'::hislip0'
        #  VISA: 'TCPIP0::'+IP_Address+'::hislip0::INSTR'
        self.jav_openvisa('TCPIP0::'+IPAddr+'::hislip0::INSTR',fily)
        # except:
        #     print('VISA Openerror.  Using Raw Socket')
        #     self.jav_openvisa('TCPIP::'+IPAddr+'::5025::SOCKET',fily)
        return self

    def jav_OpenTest(self, host):    #pylint: disable='unused-argument'
        self.debug = 0
        self.VISA  = '@py'
        # self.jav_Open(host)
        self.K2 == 'NoVISA'
        self.connected      = 1
        if self.K2 == 'NoVISA':
            mock = jaVISA_mock()
            self.jav_Open       = mock.jav_Open
            self.write          = mock.write
            self.query          = mock.query
            self.jav_Clear      = mock.jav_Clear
            self.jav_Error      = mock.jav_Error
            self.jav_read_raw   = mock.jav_read_raw
            self.jav_write_raw  = mock.jav_write_raw
            self.connected      = 0
        self.jav_ClrErr()
        self.dLastErr = ""
        return self

    def jav_openvisa(self, sVISAStr, fily=''):
        """
        TCPIP0::<IP_Address>::inst0::INSTR
        TCPIP0::<IP_Address>::hislip0::INSTR
        TCPIP0::<IP_Address>::5025::SOCKET
        GPIB::<Addr>::INSTR
        ASRL1::INSTR
        """
        TMR = timer()
        TMR.start()
        rm = pyvisa.ResourceManager(self.VISA)                          #Create Resource Manager
        TMR.tick()
        #rmList = rm.list_resources()                                   #List VISA Resources
        try:
            # self.K2 = rm.open_resource(sVISAStr, open_timeout=100)    #Create Visa Obj
            self.K2 = rm.open_resource(sVISAStr)                        #Create Visa Obj
            self.K2.timeout = 5000                                      #Timeout, millisec
            # self.K2.write_termination = self.EOL
            # self.K2.read_termination  = self.EOL
            self.jav_IDN()
            self.jav_fileout(fily, self.dataIDN)
            self.jav_ClrErr()
        except:
            if self.debug: print('jav_OpnErr: ' + sVISAStr)
            self.K2 = 'NoVISA'
        TMR.tick()
        asdf = TMR.Get_Params_Time()
        return self

    def jav_read_raw(self):
        return self.K2.read_raw()

    def jav_Reset(self):
        self.write("*RST;*CLS;*WAI")

    def jav_reslist(self):
        try:
            rm = pyvisa.ResourceManager()                           #Create Resource Manager
            rmList = rm.list_resources()                            #List VISA Resources
        except:
            rmList =["No VISA"]
        return rmList

    def jav_scpilist(self,SCPIList):
        """Send SCPI list & Query if "?" """
        ### Collect read results into a list for return.
        OutList = []
        for cmd in SCPIList:
            if cmd.find('?') == -1:
                self.write(cmd)
            else:
                ReadStr = self.query(cmd)
                OutList.append(ReadStr)
        return OutList

    def jav_Wait(self, InCMD):
        """Brute Force Wait and check *OPC? """
        start_time = time.time()
        self.write(InCMD)                                           #Initiate Command
        read = "0"
        while (int(read) & 1) != 1:                                 #Loop until done
            try:
                read = self.queryInt("*OPC?")                       #See if we can get *OPC?
            except:
                pass
            time.sleep(1)
            delta = (time.time() - start_time)
            if delta > 300:
                if self.debug: print("jav_Wai   : timeout")
                break
        if self.debug: print('jav_Wai   : %0.2fsec'%(delta))
        return delta

    def query(self,cmd):
        read ="<notRead>"
        try:
            if self.dataIDN != "":
                read = self.K2.query(cmd).strip()                   #Write if connected
        except:
            if self.debug: print("jav_RdErr : %s-->%s"%(self.Model,cmd))
        self.jav_fileout(self.f, "%s,%s,%s"%(self.Model,cmd,read))
        return read

    def queryFloat(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return [float(i) for i in strArry][0]
        except:
            return -9999.9999

    def queryFloatArry(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return [float(i) for i in strArry]
        except:
            return [-9999.9999, -888.888, -777.777, -666.666, -555.555, -444.444]

    def queryInt(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return int([float(i) for i in strArry][0])
            # Float for scientific 'e' notation
        except:
            return -9999

    def queryIntArry(self,cmd):
        try:
            strArry = self.query(cmd).split(',')
            return [int(i) for i in strArry]
        except:
            return [-9999,-8888,-7777]

    def write(self,cmd):
        try:
            if self.dataIDN != "": self.K2.write(cmd)               #Write if connected
        except:
            if self.debug: print("jav_WrtErr: %s-->%s"%(self.Model,cmd))
        self.jav_fileout(self.f, "%s,%s"%(self.Model,cmd))

    def jav_write_raw(self,SCPI):
        self.K2.write_raw(SCPI)

if __name__ == "__main__":
    RS = jaVisa()
    ipaddress   = '10.0.0.10'
    RS.debug    = 1
    # RS.jav_logscpi()
    RS.jav_Open(ipaddress)                                          #Default HiSlip
    # RS.jav_openvisa(f'TCPIP::{ipaddress}::hislip0::INSTR')        #hislip
    # RS.jav_openvisa(f'TCPIP::{ipaddress}::instr0::INSTR')         #VXI11
    # RS.jav_openvisa(f'TCPIP::{ipaddress}::5025::SOCKET')          #Socket
    rdStr = RS.query('*IDN?')
    print(rdStr)
    RS.jav_Close()



"""Common file functions"""
from datetime  import datetime
import os

class FileIO(object):
    """ Util File IO Object """
    def __init__(self):
        self.Outfile = ""
        self.sFName = ""
        self.debug = 1

    def makeFile(self,sFilepath):
        """"Create file in same directory as sFilepath."""
        BaseDir  = os.path.dirname(os.path.realpath(sFilepath))
        BaseFile = os.path.basename(sFilepath)
        OutFile  = f'{BaseDir}\\{BaseFile}'
        self.init(OutFile)
        return self

    def init(self,sName="Datalog"):
        """Append to sName file"""
        self.sFName = "%s-%s.csv"%(sName,datetime.now().strftime("%y%m%d"))
        self.Outfile = open(self.sFName, 'a')                               #Open File
        return self

    def set_filename(self,sFile):
        self.sFName = sFile
        return self

    def read(self):
        with open(self.sFName, 'r') as csv_file:
            fileData = csv_file.readlines()
        return fileData

    def readcsv(self):
        dataOut = []
        with open(self.sFName, 'r') as csv_file:
            fileData = csv_file.readlines()
            for line in fileData:
                dataOut.append(line.strip().split(','))
        return dataOut

    def readdict(self):
        """Text file split w/ space"""
        d = {}
        # BaseDir  = os.path.dirname(os.path.realpath(__file__))
        # os.chdir(BaseDir)
        # dirpath = os.getcwd()
        with open(self.sFName) as f:
            for line in f:
                # (key, val) = line.split()
                try:
                    (key, val) = line.split()
                except ValueError:
                    key = line.split(' ')[0]
                    val = line.split(' ')[1]
                d[key] = val
        return d

    def write(self,inStr):
        if self.debug: print("FileOut    : %s"%inStr)
        sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f")   #Date String
        with open(self.sFName, 'a') as csv_file:
            csv_file.write(f'{sDate},{inStr}\n')

    def write_raw(self,inStr):
        if self.debug: print("FileOut_raw : %s"%inStr)
        with open(self.sFName, 'a') as csv_file:
            csv_file.write(f'{inStr}\n')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FileIO = FileIO()
    FileIO.init("FileIO.csv")
    FileIO.write("Hello World")
    FileIO.write_raw("Hello Worldd")
    data = FileIO.readcsv()
    for i, line in enumerate(data):
        print("%d:%s"%(i,",".join(data[i])))



"""rssd.VSA.NoiseFigure_K30 test
"""
host = '192.168.1.1'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.VSA.NoiseFigure_K30     import VSA        #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSW = VSA().jav_OpenTest(host)
        self.FSW.Init_Noise()

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSW.jav_Error()[0],'0')
        self.FSW.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSW_NoiseFigure_SingleFreq(self):
        ### Must be in FreqConfig-->Tuning Mode-->Single Freq prior to test
        # self.FSW.Set_NF_FreqSweepMOde('Single')
        self.FSW.Set_NF_Single_Freq(20e9)
        self.FSW.Set_NF_Single_Meas()
        self.FSW.Set_NF_Single_Coupled_To_List('ON')
        self.FSW.Set_NF_ENR_Cal_Type('RES')

    def test_FSW_NoiseFigure_Sweep(self):
        ### Must be in FreqConfig-->Tuning Mode-->Sweep prior to test
        ### ENR Settings
        # self.FSW.Set_NF_ENR_Cal_Type('DIOD')
        self.FSW.Set_NF_ENR_Meas_Type('DIOD')
        self.FSW.Set_NF_ENR_Meas_Mode('TABL')
        self.FSW.Set_NF_ENR_Temp(0)
        self.FSW.Set_NF_ENR_Table('DEFAULT')
        self.FSW.Set_NF_Cal_Type("AUTO")
        self.FSW.Config_NF_Cal('HOT')
        self.FSW.Set_NF_2ndCorr_State('OFF')
        # self.FSW.Set_NF_Cal_State('ON')
        # self.FSW.Set_NF_Sweep('SING')
        self.FSW.Set_NF_DUT_InLoss_Mode("SPOT")
        self.FSW.Set_NF_DUT_InLoss_TableName('DEFAULT')
        self.FSW.Set_NF_DUT_InLoss_Table('1MHz,10,2MHz,12')
        self.FSW.Set_NF_DUT_OutLoss_Mode('SPOT')
        self.FSW.Set_NF_DUT_OutLoss_TableName('DEFAULT')
        self.FSW.Set_NF_DUT_OutLoss_Table('1MHz,10,2MHz,12')

    def test_FSW_NF_Get_Default(self):
        nullVal = self.FSW.Get_NF_Gain()
        nullVal = self.FSW.Get_NoiseFigure()
        nullVal = self.FSW.Get_NoiseTemp()
        nullVal = self.FSW.Get_NF_PHot()
        nullVal = self.FSW.Get_NF_PCold()

    def test_FSW_NF_Get_Extra(self):
        nullVal = self.FSW.Get_NF_CalCold()
        nullVal = self.FSW.Get_NF_CalHot()
        self.FSW.Get_YFactor()

    def test_FSW_NF_States(self):
        self.FSW.Set_NF_2ndCorr_State(1)
        self.FSW.Set_NF_2ndCorr_State(0)
        self.FSW.Set_NF_Cal_State(1)
        self.FSW.Set_NF_Cal_State(0)
        self.FSW.Set_NF_Single_Coupled_To_List(1)
        self.FSW.Set_NF_Single_Coupled_To_List(0)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""Mock Yet(Just) Another VISA wrapper"""
#pylint: disable=R0201,W0613
class jaVISA_mock(object):                          #pylint: disable=R0205
    """Instrument Common functions"""
    def __init__(self):
        self.dataIDN    = ""    # Raw IDN String
        self.Make       = ""    # IDN Make
        self.Model      = ""    # IDN Model
        self.Device     = ""    # IDN Device
        self.Version    = ""    # IDN Version
        self.debug      = 1     # Print or not
        self.EOL        = '\n'
        self.f          = ''    # file log object

    def delay(self,sec):
        pass

    def jav_Clear(self):
        pass

    def jav_Close(self):
        pass

    def jav_ClrErr(self):
        return "no Error"

    def jav_Error(self):
        return '0,No Error'

    def jav_IDN(self,prnt=1):                       #pragma: no cover
        self.dataIDN    = "test"                    #Temp for self.query
        self.Make       = 'MakeTest'
        self.Model      = 'ModelTest'
        self.Device     = 'DevTest'
        self.Version    = '1.0.0'
        if self.debug: print('jav_IDN    : %s'%(self.dataIDN))
        return self.dataIDN

    def jav_OPC_Wait(self, InCMD):
        return 0.0

    def jav_Open(self, IPAddr, fily=''):
        return self

    def jav_openvisa(self, sVISAStr, fily=''):
        return self

    def jav_read_raw(self):
        return b'1234567890'

    def jav_write_raw(self,SCPI):
        pass

    def query(self,cmd):
        return "<notRead>"

    def write(self,cmd):
        pass

if __name__ == "__main__":
    RS = jaVISA_mock()
    RS.jav_openvisa(f'TCPIP::192.168.1.100::200::SOCKET')    #Socket
    print(RS.query("*IDN?"))
    RS.jav_Close()



"""
Purpose: Vector Signal Generator 802.11 methods

Standrd |Modu|Freqncy|Mode|PPDU|BW |MIMO|BitRate|Opt
--------|----|-------|----|----|---|----|-------|----
802.11b  DSSS 2.4     Legy  CCK 20 SISO   11Mbps K54
802.11a  OFDM     5.4 Legy         SISO   54Mbps K54
802.11g  both 2.4 5.4 Legy   L     SISO   54Mbps K54
802.11n  OFDM 2.4 5.4 GrnF  HT  40 MIMO  300Mbps K54
802.11ac OFDM     5.4 MixM VHT 160 MIMO 1750Mbps K86
802.11ax OFDM 2.4 5.4 MixM  HE 160 MIMO   11Gbps K142
802.11be OFDM 2.4 5.4 MixM EHT 320 MIMO     Gbps K147
"""
from rssd.VSG.Common import VSG     #pylint: disable=E0611,E0401

class VSG(VSG):                     #pylint: disable=E0102
    """ Rohde & Schwarz Vector Signal Generator 802.11 Object """
    def __init__(self):
        super(VSG,self).__init__()    #Python2/3
        self.Model = "SMW"
        self.WLAN_Std  = 'N'
        self.WLAN_ChBW = 20         #MHz
        self.WLAN_MCS  = 1

    #####################################################################
    ### VSG Query
    #####################################################################
    def Get_WLAN_ChBW(self):
        rdStr = self.query(f':SOUR:BB:WLNN:BW?')
        return rdStr

    def Get_WLAN_MCS(self):
        #WLAN-->Frame Blocks-->PPDU Conf..--> MCS Config
        rdStr = self.query(f':SOUR:BB:WLNN:FBL1:MCS?')
        return rdStr

    def Get_WLAN_Modulation(self):
        #WLAN-->Frame Blocks-->PPDU Conf..--> MCS Config
        rdStr = self.query(f':SOUR:BB:WLNN:FBL1:MOD1?')
        return rdStr

    def Get_WLAN_PPDU(self):
        #n : HT
        #ac: VHT
        #ax: HE
        rdStr = self.query(f':SOUR:BB:WLNN:FBL1:TMOD?')
        return rdStr

    def Get_WLAN_Standard(self):
        rdStr = self.query(f':SOUR:BB:WLNN:FBL1:TMOD?')
        if rdStr[0] == 'L':                                     #pylint: disable=R1705
            return 'A'
        elif (rdStr[:3] == 'CCK') or (rdStr[:3] == 'PBC'):
            return 'B'
        elif rdStr[:2] == 'HT':
            return 'N'
        elif rdStr[:1] == 'V':
            return 'AC'
        elif rdStr[:2] == 'HE':
            return 'AX'
        else:
            return 'Error'

    #####################################################################
    ### VSG Setting
    #####################################################################
    def Set_WLAN_BBState(self, sState):
        if (sState == 1) or (sState == 'ON'):
            self.jav_OPC_Wait(':SOUR1:BB:WLNN:STAT 1')
        else:
            self.write(':SOUR1:BB:WLNN:STAT 0')

    def Set_WLAN_ChBW(self,iChBW):
        self.write(f':SOUR:BB:WLNN:BW BW{iChBW:02d}')
        self.WLAN_ChBW = iChBW

    def Set_WLAN_MCS(self,iMCS):
        #WLAN-->Frame Blocks-->PPDU Conf..--> MCS Config
        self.write(f':SOUR:BB:WLNN:FBL1:MCS MCS{iMCS}')

    def Set_WLAN_Modulation(self, sMod):
        """QPSK; QAM16; QAM64; QAM256"""
        #WLAN-->Frame Blocks-->PPDU Conf..--> MCS Config
        self.write(f':SOUR:BB:WLNN:FBL1:MOD1 {sMod}')

    def Set_WLAN_Standard(self, sStd):
        """B G A N AC"""
        #WLAN-->Frame Blocks-->TxMode
        sStd.upper()
        if sStd == 'B':
            self.write(f':SOUR:BB:WLNN:FBL1:PMOD LEG')  #Set Physical Mode
            self.write(f':SOUR:BB:WLNN:FBL1:TMOD CCK')
        elif sStd == 'G' or sStd == 'A':
            self.write(f':SOUR:BB:WLNN:FBL1:PMOD LEG')  #Set Physical Mode
            self.write(f':SOUR:BB:WLNN:FBL1:TMOD L{self.WLAN_ChBW}')
        elif sStd == 'N':
            self.write(f':SOUR:BB:WLNN:FBL1:PMOD MIX')  #Set Physical Mode
            self.write(f':SOUR:BB:WLNN:FBL1:TMOD HT{self.WLAN_ChBW}')
        elif sStd == 'AC':
            self.write(f':SOUR:BB:WLNN:FBL1:PMOD MIX')  #Set Physical Mode
            self.write(f':SOUR:BB:WLNN:FBL1:TMOD V{self.WLAN_ChBW}')
        elif sStd == 'AX':
            self.write(f':SOUR:BB:WLNN:FBL1:PMOD MIX')  #Set Physical Mode
            self.write(f':SOUR:BB:WLNN:FBL1:TMOD HE{self.WLAN_ChBW}')
        else:
            print(f'Set_WLAN_Standard: {sStd} not supported')


#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    WLAN = VSG()
    WLAN.jav_Open("192.168.1.114")
    print(WLAN.Get_WLAN_Standard())
    WLAN.jav_Close()



"""Vector Signal Generator Common Functions
"""
from time           import sleep
from rssd.yaVISA    import jaVisa        #pylint: disable=E0611,E0401

class VSG(jaVisa):
    """ Rohde & Schwarz Vector Signal Generator Object """
    def __init__(self):
        super(VSG,self).__init__()     #Python2/3
        self.Model  = "SMW"
        self.NRP    = 2

    #####################################################################
    ### SMW Get
    #####################################################################
    def Get_ArbClockFreq(self):
        SCPI = self.queryFloat('SOUR:BB:ARB:CLOC?')
        return SCPI

    def Get_ArbName(self):
        WvName = self.query('BB:ARB:WAV:SEL?')
        return WvName

    def Get_ArbTime(self):
        Fs = self.Get_ArbClockFreq()
        Points = self.queryInt('BB:ARB:WAV:POIN?')
        WvTime = Points/int(Fs)
        return WvTime

    def Get_ArbInfo(self):
        ClkFreq = self.Get_ArbClockFreq()
        ArbName = self.Get_ArbName()
        ArbTime = self.Get_ArbTime()
        return f'{ClkFreq},{ArbName},{ArbTime}'

    def Get_CrestFactor(self):
        PEP = self.Get_PowerPEP()
        RMS = self.Get_PowerRMS()
        return PEP - RMS

    def Get_Freq(self):
        rdStr = self.queryFloat(':SOUR1:FREQ:CW?')     #RF Freq
        return rdStr

    def Get_ListMode_IndexCurr(self):
        rdInt = self.queryInt('SOUR1:LIST:IND?')
        return rdInt

    def Get_ListMode_IndexStop(self):
        rdInt = self.queryInt('SOUR1:LIST:IND:STOP?')
        return rdInt

    def Get_NRPPower(self,NRP=2):
        self.write(f':INIT{NRP}:POW:CONT 1')
        self.write(f'SENS{NRP}:UNIT DBM')
        self.query(f'SENS{NRP}:TYPE?')
        SCPI = self.queryFloat(f':READ{NRP}:POW?')
        if SCPI < -198: SCPI = self.queryFloat(f':READ{NRP}:POW?')
        if SCPI < -198: SCPI = self.queryFloat(f':READ{NRP}:POW?')
        return SCPI

    def Get_OS_Dir(self):
        rdStr = self.query(f'MMEM:CDIR?')
        return rdStr

    def Get_OS_FileList(self,filterKey=''):
        """Return list of filenames in sDir"""
        rdStr   = self.query('MMEM:CATalog?').split('","')
        rdStr   = rdStr[2:]                                     #Filter out . and ..
        outList = []
        for i, file in enumerate(rdStr):                        #pylint: disable=W0612
            rdStr[i] = rdStr[i].split(',')
            if rdStr[i][0].find(filterKey) > -1:
                outList.append(rdStr[i][0])
        return outList

    def Get_PowerPEP(self,RF=1):
        SCPI = self.queryFloat('SOUR%d:POW:PEP?'%RF)
        return SCPI

    def Get_PowerRMS(self,RF=1):
        SCPI = self.queryFloat('SOUR%d:POW?'%RF)
        return SCPI

    def Get_PowerInfo(self):
        PEP = self.Get_PowerPEP()
        RMS = self.Get_PowerRMS()
        CRS = PEP - RMS
        return f'{PEP},{RMS},{CRS}'

    def Get_SysC_All(self):
        BBSour  = self.Get_SysC_BBSource()
        Fading  = self.Get_SysC_Fading()
        Mode    = self.Get_SysC_Mode()
        rdStr = f'{Mode},{Fading},{BBSour}'
        print(rdStr)
        return rdStr

    def Get_SysC_BBSource(self):
        rdStr = self.query('SCON:BAS:SOUR?')
        return rdStr

    def Get_SysC_Fading(self):
        rdStr = self.query('SCON:FAD?')
        return rdStr

    def Get_SysC_Mode(self):
        rdStr = self.query('SCON:MODE?')
        return rdStr

    #####################################################################
    ### SMW INIT
    #####################################################################
    def Init_Wideband(self):
        self.write('SOUR:POW:ATT:DIG 3')            #Set Digital Attenuation
        self.write('POW:ALC:STATE AUTO')            #Turn ALC ON|OFF|OFFT|AUTO|
        self.write('SOUR:POW:ALC:DAMP AUTO')        #Turn Driver AMP ON|OFF|AUTO
        self.write('SOUR:BB:IQG DB8')               #Baseband IQ gain

        ## Not so critical
        self.write('SOUR:AWGN:STAT 0')              #Turn AWGN off (default)
        self.write('BBIN:STAT OFF')                 #Turn BB Input off(default)

    #####################################################################
    ### SMW Settting Methods
    #####################################################################
    # def Set_IQ_Data(self):
    #     IData = [0.1,0.2,0.3]
    #     QData = [0.4,0.5,0.6]

    #     ### ASCII
    #     scpi  = ':MMEM:DATA:UNPR "NVWFM://var//user//wave.wv",#'        # Ascii Cmd
    #     iqsize= str(len(IData)*4)                                       # Calculate bytes of IQ data
    #     scpi  = scpi + str(len(iqsize)) + iqsize                        # Calculate length of iqsize string
    #     ### Binary
    #     iqdata= np.vstack((IData,QData)).reshape((-1,),order='F')       # Combine I&Q Data
    #     bits  = np.array(iqdata*32767, dtype='>i2')                     # Convert to big-endian 2byte int
    #     ### ASCII + Binary
    #     cmd   = bytes(scpi, 'utf-8') + bits.tostring()                  # Add ASCII + Bin
    #     self.K2.write_raw(cmd)
    #     self.write('SOUR1:BB:ARB:WAV:CLOC "/var/user/wave.wv",1.1E6')    # Set Fs/Clk Rate
    #     self.write('BB:ARB:WAV:SEL "/var/user/wave.wv"')                 # Select Arb File

    def Set_ALC_RFDriveAmp(self,sState):
        """input: ON, OFF, AUTO, FIX """
        self.query('SOUR:POW:ALC:DAMP %s;*OPC?'%sState)

    def Set_ArbClockFreq(self,fFreq,RF=1):
        self.write('SOUR%d:BB:ARB:CLOC %f'%(RF,fFreq))

    def Set_ArbNextSeg(self,num):
        self.query('BB:ARB:WSEG:NEXT %d;*OPC?'%(num))

    def Set_ArbSeg(self,Seg):
        self.write('SOUR:BB:ARB:WSEG:NEXT %d'%Seg)
        #  self.write('SOUR:BB:ARB:WSEG:NEXT:EXEC')

    def Set_ArbState(self,sState):
        if sState in (1,'1','ON'):
            self.query(':SOUR1:BB:ARB:STAT 1;*OPC?')
        elif sState in (0,'0','OFF'):
            self.query(':SOUR1:BB:ARB:STAT 0;*OPC?')

    def Set_ArbWv(self,InWv):
        self.query('BB:ARB:WAV:SEL "%s"; *OPC?'%InWv)

    def Set_BBState(self,sState):
        """'ON' 'OFF' 1 or 0"""
        if sState in (1,'1','ON'):
            self.query(':SOUR1:BB:ARB:STAT 1;*OPC?')
        elif sState in (0,'0','OFF'):
            self.query(':SOUR1:BB:ARB:STAT 0;*OPC?')

    def Set_Freq(self,freq):
        """Unit: Hz"""
        self.write(f':SOUR1:FREQ:CW {freq}')     #RF Freq

    def Set_IQMod(self,sState):
        """input: ON, OFF """
        if sState in (1,'1','ON'):
            self.query('SOUR:IQ:STAT ON;*OPC?')
        elif sState in (0,'0','OFF'):
            self.query('SOUR:IQ:STAT OFF;*OPC?')

    def Set_ListMode_Dwell(self, sec):
        self.write(f'SOUR1:LIST:DWEL {sec}')

    def Set_ListMode_File(self,sFile):
        if sFile.find('.lsw') == -1:
            sFile = sFile + '.lsw'
        self.write(f'SOUR1:LIST:SEL "/var/user/{sFile}"')

    def Set_ListMode(self,sState):
        """input: LIST CW"""
        self.query(f':SOUR1:FREQ:MODE {sState};*OPC?')

    def Set_ListMode_TrigExecute(self):
        self.query(':SOUR1:LIST:TRIG:EXEC;*OPC?')

    def Set_ListMode_TrigSource(self,sSource):
        """SING AUTO STEP ESTEP ESING"""
        # USER5 Valid Signal A
        # USER6 Valid SIgnal B

        if 'AUTO' in sSource:
            self.write(f'SOUR1:LIST:MODE AUTO')
            self.write(f'SOUR1:LIST:TRIG:SOUR AUTO')
        elif 'SING' in sSource:
            self.write(f'SOUR1:LIST:MODE AUTO')
            self.write(f'SOUR1:LIST:TRIG:SOUR SING')
        elif 'STEP' in sSource:
            self.write(f'SOUR1:LIST:MODE STEP')
            self.write(f'SOUR1:LIST:TRIG:SOUR SING')
        elif 'ESING' in sSource:
            self.write(f'SOUR1:LIST:MODE AUTO')
            self.write(f'SOUR1:LIST:TRIG:SOUR EXT')
        elif 'ESTEP' in sSource:
            self.write(f'SOUR1:LIST:MODE STEP')
            self.write(f'SOUR1:LIST:TRIG:SOUR EXT')

    def Set_ListMode_TrigWait(self):
        indx = 0
        stop = self.Get_ListMode_IndexStop()
        while indx != stop:
            sleep(0.1)
            indx = self.Get_ListMode_IndexCurr()

    def Set_ListMode_RMode(self, sMode):
        """RunMode: LIVE LEARned """
        self.write(f'SOUR:LIST:RMOD {sMode}')

    def Set_NRP_Freq(self, Freq):
        """Frequency if NRP in USER mode"""
        self.write(f':SENS{self.NRP}:POW:FREQ {Freq}')

    def Set_NRP_Mode(self, sMode):
        """USER | A"""
        if sMode == "USER":
            self.write(f':SENS{self.NRP}:POW:SOUR USER')
        else:
            self.write(f':SENS{self.NRP}:POW:SOUR A')

    def Set_OS_Dir(self,sDir):
        self.write(f'MMEMory:CDIRectory "/var/user/{sDir}"')

    def Set_OptimizeAll(self):
        """Please set"""
        self.Set_OptimizeIQ()
        self.Set_OptimizeLevel()

    def Set_OptimizeIQ(self):
        """Please set"""
        self.K2.timeout = 9000
        self.query(':CAL:IQM:LOC?')       # SMW IQ Adjust

    def Set_OptimizeLevel(self):
        """Please set"""
        self.K2.timeout = 9000
        self.query(':CAL:LEV:LOC?')       # SMW Level Adjust

    def Set_PhaseDelta(self,fPhase):
        self.write(':SOUR1:PHASE %d'%(fPhase))

    def Set_PM_State(self, State):
        if State in (1, '1', 'ON'):
            self.write(':SOUR1:PM1:STAT ON')
        else:
            self.write(':SOUR1:PM1:STAT OFF')

    def Set_PM_Source(self, Source):
        """LF1 LF2 EXT1 EXT2 NOIS INT EXT"""
        self.write(f':SOUR1:PM1:SOUR {Source}')

    def Set_Ref_Source(self,sSource):
        """Ext Int"""
        self.write(f':SOUR1:ROSC:SOUR {sSource}')

    def Set_Ref_Freq(self,sFreq):
        """10MHZ 100MHZ 1GHz"""
        self.write(f':SOUR1:ROSC:EXT:FREQ {sFreq}')

    def Set_Ref_SyncBW(self,sBW):
        """WIDE NARR"""
        self.write(f':SOUR1:ROSC:EXT:SBAN {sBW}')    #SMW WIDE(Vary less)|NARR(varys more) bandwidth

    def Set_RFPwr(self,fPow):
        self.write('SOUR:POW %f'%fPow)

    def Set_RFState(self,sState):
        ''' 0 1 '''
        if sState in (1,'1','ON'):
            self.query('OUTP 1;*OPC?')
        if sState in (0,'0','OFF'):
            self.query('OUTP 0;*OPC?')

    def Set_Setting(self,sSettingFile):
        """*.savrcltxt SettingFile"""
        self.jav_OPC_Wait(f'SYST:RCL "/var/user/{sSettingFile}"')

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    # this won't be run when imported
    SMW = VSG().jav_Open("192.168.1.114")
    getVal = SMW.Get_OS_FileList('savrcltxt')
    SMW.Set_Setting('test.savrcltxt')



"""Test rssd.instrument"""
ipaddress = '192.168.58.115'

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.instrument import instr                   #pylint:disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #Run before each test
        print("",end="")
        self.instr = instr()

    def tearDown(self):                             #Run after each test
        print(self.instr.Model)
        self.instr.close()

###############################################################################
### <Test>
###############################################################################
    def test_open_default(self):
        self.instr.open(ipaddress)

    def test_open_hislip(self):
        self.instr.open(ipaddress, type='hislip')

    def test_open_socket(self):
        self.instr.open(ipaddress, type='socket')

    def test_open_test(self):
        self.instr.open(ipaddress, type='test')

    def test_open_visa_socket(self):
        self.instr.open(ipaddress, type='visa-socket')

    def test_open_vxi11(self):
        self.instr.open(ipaddress, type='vxi11')

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':                                  # pragma: no cover
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



""" Radio Communication Tester(RCT) Functions"""
from rssd.yaVISA import jaVisa              #pylint: disable=E0611,E0401

class RCT(jaVisa):
    """ Rohde & Schwarz Base Station Emulator Object """
    def __init__(self):
        super(RCT, self).__init__()
        self.Model = "CMW-GPRF"

    ###########################################################################
    ### BSE Get Functions
    ###########################################################################
    def Get_Gen_ArbWv(self):                                                    #val
        rdStr = self.query('SOUR:GPRF:GEN:ARB:FILE?')
        return rdStr

    def Get_Gen_Freq(self):                                                     #val
        rdStr = self.query('SOUR:GPRF:GEN:RFS:FREQ?')
        return rdStr

    def Get_Gen_Mode(self):                                                     #val
        rdStr = self.query('SOUR:GPRF:GEN:BBM?')
        return rdStr

    def Get_Gen_Port(self):                                                     #val
        rdStr = self.query('ROUT:GPRF:GEN:SPAT?')
        return rdStr

    def Get_Meas_Power(self):                                                   #Val
        rdStr   = -9999
        num     = 0
        while (rdStr == -9999) and (num < 10):
            num += 1
            try:
                self.write('INIT:GPRF:MEAS:POW')                                #RUN state
                self.query('*OPC?')
                rdStr = self.queryFloatArry('FETC:GPRF:MEAS:POW:AVER?')[1]      # Doesnt Clr Memory
                # rdStr = self.query('READ:GPRF:MEAS:POW:AVER?')                # Clears Memory
                # rdStr = self.query('CALC:GPRF:MEAS:POW:AVER?')                # Chk Limits
            except:
                print(f'CMP.Get_Meas_Power{num}')
        return rdStr

    def Get_Meas_Port(self):                                                    #Val
        rdStr = self.query('ROUT:GPRF:MEAS:SPAT?')
        return rdStr

    def Get_Options(self):
        rdStr = self.query('SYST:BASE:OPT:LIST?').split(',')
        return rdStr

    ###########################################################################
    ### BSE Init Functions
    ###########################################################################
    def Init_Syst(self):
        self.write('SYST:GEN:ALL:OFF')
        self.write('SYST:MEAS:ALL:OFF')

    ###########################################################################
    ### RCT Init Functions
    ###########################################################################
    def Init_Gen(self,port=1):                                                  #Val
        # self.write('ROUT:GPRF:GEN:SCENario:SALone R118, TX11')
        self.Set_Gen_Port(port)
        self.write('CONF:GPRF:GEN:CMWS:USAGe:TX:ALL R118, OFF, OFF,OFF, OFF, OFF, OFF, OFF, OFF')

    def Init_Meas_FFT(self,port=1):                                             #Val
        # self.write('ROUT:GPRF:MEAS:SCEN:SAL R1{port}, RX1'%port)
        self.Set_Meas_Port(port)
        self.write('INIT:GPRF:MEAS:FFTS')

    def Init_Meas_Power(self,port=1):                                           #Val
        # self.write('ROUT:GPRF:MEAS:SCEN:SAL R1{port}, RX1'%port)
        self.Set_Meas_Port(port)
        self.write('FORMAT:BASE:DATA ASCII')
        self.write('INIT:GPRF:MEAS:POW')
        self.write('CONF:GPRF:MEAS:POW:MODE POW')      #POW|STAT

    ###########################################################################
    ### RCT INIT Functions
    ###########################################################################
    def Set_Gen_ArbExec(self):
        self.query('TRIG:GPRF:GEN:ARB:MAN:EXEC')

    def Set_Gen_ArbWv(self,sName):
        self.write(f":SOUR:GPRF:GEN:ARB:FILE '@Waveform/{sName}'")

    def Set_Gen_Freq(self,freq):
        self.write(f'SOUR:GPRF:GEN:RFS:FREQ {freq}')

    def Set_Gen_ListMode(self,sState):
        if sState.upper() == "ON":
            self.query('SOUR:GPRF:GEN:LIST ON;*OPC?')
        else:
            self.query('SOUR:GPRF:GEN:LIST OFF;*OPC?')

    def Set_Gen_Mode(self, mode):
        """ 'CW', 'ARB' or 'DTONE' """
        self.write(f'SOUR:GPRF:GEN:BBM {mode}')

    def Set_Gen_Port(self, port):                                               #Val
        """ CMP: 'P<x>.IFOut' | 'P<x>.RRH.RF<y>'"""
        self.write(f'ROUT:GPRF:GEN:SPAT "{port}"')

    def Set_Gen_Port_State(self,port=1,state='ON'):                             #val
        """ 'ON' 'OFF' """
        if state == 'ON':
            self.write(f'CONFigure:GPRF:GEN:CMWS:USAGe:TX R1{port}, ON')
        else:
            self.write(f'CONFigure:GPRF:GEN:CMWS:USAGe:TX R1{port}, OFF')

    def Set_Gen_RFPwr(self,fPwr):                                               #Val
        self.write(f'SOUR:GPRF:GEN:RFS:LEV {fPwr}')

    def Set_Gen_RFState(self,sState):                                           #Val
        """ ON | OFF """
        if sState.upper() == "ON":
            self.query('SOUR:GPRF:GEN:STAT ON;*OPC?')
        else:
            self.query('SOUR:GPRF:GEN:STAT OFF;*OPC?')

    def Set_Meas_Autolevel(self):
        """ExpNomPwr=MeasPwr and UserMargin=0"""
        self.Init_Meas_Power()
        self.Set_Meas_UserMargin(0)
        self.Set_Meas_Expected_Nom_Power(40)
        pwr = self.Get_Meas_Power()
        print(pwr)
        self.Set_Meas_Expected_Nom_Power(pwr + 2)

    def Set_Meas_RFBW(self,fFreq):
        """Expected RF BW"""
        self.write(f'CONF:GPRF:MEAS:POW:FILT:TYPE BAND')
        self.write(f'CONF:GPRF:MEAS:POW:FILT:BAND:BWID  {fFreq}')

    def Set_Meas_Expected_Nom_Power(self,fPwr):
        """Expected Nominal Power (RMS)"""
        self.write(f'CONFigure:GPRF:MEAS:RFSettings:ENPower {fPwr}')

    def Set_Meas_Freq(self,fFreq):                                              #Val
        # self.write('CONF:GPRF:MEAS:SPEC:FREQ:CENT %d'%fFreq)
        self.write(f'CONF:GPRF:MEAS:RFS:FREQ {fFreq}')

    # def Set_Meas_InitImm(self):
    #     #self.query('INIT:GPRF:MEAS:SPEC;*OPC?')
    #     self.query('INIT:GPRF:MEAS:;*OPC?')

    def Set_Meas_Port(self, port):
        """ CMP: 'P<x>.IFIn' | 'P<x>.RRH.RF<y>'"""
        self.write(f'ROUT:GPRF:MEAS:SPAT "{port}"')

    def Set_Meas_Pwr_MLength(self,length):
        self.write(f'CONF:GPRF:MEAS:POW:SLEN {length}')
        self.write(f'CONF:GPRF:MEAS:POW:MLEN {length}')

    def Set_Meas_RefLevl(self,fRefLvl):                                         #Val
        """ENP = Expected Nominal Power"""
        self.Set_Meas_Expected_Nom_Power(fRefLvl)

    def Set_Meas_Span(self,fFreq):
        """ 10; 20; 40; 80; 160; 250; 500; 1000 MHz allowed """
        self.write(f'CONF:GPRF:MEAS:FFTS:FSP {fFreq}')

    def Set_Meas_SweepTime(self,fTime):
        if fTime > 0:
            self.write(f'CONF:GPRF:MEAS:SPEC:FSW:SWT {fTime}')
        else:
            self.write(f'CONF:GPRF:MEAS:SPEC:FSW:SWT:AUTO ON')

    def Set_Meas_TriggerSource(self,source):
        """string: IF Power Free Run """
        self.write(f'TRIG:GPRF:MEAS:POW:SOUR "{source}"')

    def Set_Meas_TriggerThreshold(self,num):
        """ -50 to 0 """
        self.write(f'TRIG:GPRF:MEAS:POW:THR {num}')

    def Set_Meas_UserMargin(self,fPwr):
        """User Margin (Crest Factor)"""
        self.write(f'CONFigure:GPRF:MEAS:RFSettings:UMARgin {fPwr}')

    def Set_Sys_DisplayUpdate(self,state):
        self.write('SYST:DISP:UPD %s'%state)                                #Display Update State

    def Set_Sys_RxPortLoss(self,port=1,fLoss=0):                            #Val
        self.write(f"CONF:CMWS:FDC:DEAC:RX R1{port}")
        self.write(f"CONF:BASE:FDC:CTAB:CRE 'In{port}',70.0e6,{fLoss},6000.e6,{fLoss}")
        self.write(f"CONF:CMWS:FDC:ACT:RX R1{port},'In{port}'")

    def Set_Sys_TxPortLoss(self,port=1,fLoss=0):                            #Val
        #CONF:BASE:FDC:CTAB:CRE 'downlink', 500.0e6,0.5, 1000e6,1.0, 1800.e6,1.5, 2300.e6,1.8, 5000.e6,2.5,  6000.e6,2.8
        self.write(f"CONF:CMWS:FDC:DEAC:TX R1{port}")
        self.write(f"CONF:BASE:FDC:CTAB:CRE 'Out{port}',70.0e6,{fLoss},6000.e6,{fLoss}")
        self.write(f"CONF:CMWS:FDC:ACT:TX R1{port},'Out{port}'")

###############################################################################
### Run if Main
###############################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    CMW = RCT()
    CMW.jav_Open("192.168.1.160")
    print(CMW.Get_Meas_Power())
    CMW.jav_Close()



import pysftp
cnopts = pysftp.CnOpts()
cnopts.hostkeys = None

myHostname = "192.168.1.160"
myUsername = "root"
myPassword = "iluvr0hde"
remotePath = '/home/instrument/fw/data/iqrecorder/'
localPath  = 'C:\\Users\\lim_m\\Desktop\\CMPTEstData\\'
filename   = 'IQFile.iqw'

with pysftp.Connection(host=myHostname, username=myUsername, password=myPassword,cnopts=cnopts) as sftp:
    print("Connection succesfully established ... ")

    sftp.get(remotePath+filename, localPath+filename)
    # sftp.cwd(remotePath)
    # data = sftp.listdir_attr()            # Get the directory and file listing

# for i in data:                  # Prints out the directories and files, line by line
#     print(f'{i.filename} {i.st_size}')



""" OTA Common Functions

Terms:  Azimuth:    Theta; Turntable;
        Elevation:  Phi; ATS1000 Arm;
OTA:    ATS1000     Great Circle Cut; Turntable & Elevation Arm
        ATS1500     Az over El;
        ATS1800     Az over El;
"""
from rssd.instrument import instr

class OTA(instr):
    """ Rohde & Schwarz Over The Air Chamber Object """
    def __init__(self):
        super(OTA, self).__init__()
        self.Model      = "OTA"
        self.EOL        = '\x00'
        self.cmdWait    = 0.05     #Seconds

    def query(self,cmd):                            #pragma: no cover
        self.bus.EOL = self.EOL
        read = self.bus.query(cmd + self.EOL).strip()
        return read[:-1]

###############################################################################
### Debug Main.  Won't run when imported
###############################################################################
if __name__ == "__main__":
    ATS1000 = OTA()
    ATS1000.open('169.254.2.10', type = 'socket', param = 200)
    print(ATS1000.Get_IDN())



"""Vector Signal Analyzer Noise Figure K30 Functions"""
from rssd.VSA.Common import VSA

class VSA(VSA):
    """ Rohde & Schwarz Vector Signal Analyzer Noise Figure Object """
    def __init__(self):
        super(VSA, self).__init__()
        self.Model = "FSW"

    ###########################################################################
    ### FSW V5G
    ###########################################################################
    def Init_Noise(self):
        self.Set_Channel('NOISE')

    ###########################################################################
    ### FSW ENR & Noise (With Option Installed)
    ###########################################################################
    def Set_NF_ENR_Cal_Type(self, sENRtype):
        """ DIOD | RES (Noise Diode or Resistor)"""
        self.write(':SENS:CORR:ENR:COMM OFF')
        self.write(':SENS:CORR:ENR:CAL:TYPE %s'%sENRtype)

    def Set_NF_ENR_Meas_Type(self, sENRtype):
        """ DIOD | RES (Noise Diode or Resistor)"""
        self.write('SENS:CORR:ENR:MEAS:TYPE %s'%sENRtype)

    def Set_NF_ENR_Meas_Mode(self, sENRmode):
        """TABL """
        self.write('SENS:CORR:ENR:MEAS:MODE %s'%sENRmode)

    def Set_NF_ENR_Temp(self, fnoiseTemp):
        """Temp in Kelvin = C + 273.15"""
        self.write(':SENS:CORR:TEMP:CONT MAN')
        self.write(':SENS:CORR:TEMP %f'%fnoiseTemp)

    def Set_NF_ENR_Table(self, sENRtable):
        """Table Name"""
        self.write(":SENS:CORR:ENR:MEAS:TABL:SEL '%s'"%sENRtable)     #'346B.1321'

    def Set_NF_Cal_Type(self, snoiseCalType):
        """AUTO | MAN Measurement Mode"""
        self.write('CONF:CONT %s'%snoiseCalType)

    def Config_NF_Cal(self, snoiseCalTemp):
        """HOT|COLD"""
        self.write('CONF:MEAS %s'%snoiseCalTemp)

    def Set_NF_2ndCorr_State(self, sState):
        """ON|OFF|1|0"""
        if sState in (1, '1', 'ON'):
            self.write('SENS:CORR:STAT ON')
        if sState in (0, '0', 'OFF'):
            self.write('SENS:CORR:STAT OFF')

    def Set_NF_Cal_State(self, sState):
        """ON|OFF|1|0"""
        if sState in (1, '1', 'ON'):
            self.write('SENS:CORR:STAT ON;*WAI')
        if sState in (0, '0', 'OFF'):
            self.write('SENS:CORR:STAT OFF;*WAI')

    def Set_NF_Sweep(self, sNoiseSweep):
        """SING|CONT"""
        self.write('SENS:CONF:LIST %s'%sNoiseSweep)
        self.Set_InitImm()

    def Set_NF_Single_Meas(self):
        """perform a single frequency measurement"""
        self.write(f'CONF:FREQ:SING')

    def Set_NF_Single_Coupled_To_List(self, sState):
        """ON|OFF|1|0"""
        if sState in (1, '1', 'ON'):
            self.write('FREQ:SING:COUP ON')
        if sState in (0, '0', 'OFF'):
            self.write('FREQ:SING:COUP OFF')

    def Set_NF_Single_Freq(self, fFreq):
        """Frequency in Single Sweep Mode"""
        self.write(f'FREQ:SING {fFreq:.0f}')

    def Set_NF_DUT_InLoss_Mode(self, sinputMode):
        """SPOT | TABL"""
        self.write('CORR:LOSS:INP:MODE %s'%sinputMode)

    def Set_NF_DUT_InLoss_TableName(self, sinTableName):
        self.write("CORR:LOSS:INP:TABL:SEL '%s'"%sinTableName)

    def Set_NF_DUT_InLoss_Table(self, sinputTable):
        """Enter loss table as csv: '1MHz,10,2MHz,12'"""
        self.write('CORR:LOSS:INP:TABL %s'%sinputTable)

    def Set_NF_DUT_OutLoss_Mode(self, soutputMode):
        """SPOT | TABL"""
        self.write('CORR:LOSS:OUTP:MODE %s'%soutputMode)

    def Set_NF_DUT_OutLoss_TableName(self, soutTableName):
        self.write("CORR:LOSS:OUTP:TABL:SEL '%s'"%soutTableName)

    def Set_NF_DUT_OutLoss_Table(self, soutputTable):
        """Enter loss table as csv: '1MHz,10,2MHz,12'"""
        self.write('CORR:LOSS:OUTP:TABL %s'%soutputTable)

    ###########################################################################
    ### Retrieve Measurements
    ###########################################################################
    def Get_NF_Gain(self):
        Gain = self.queryFloat('TRAC? TRACE1, GAIN')
        return Gain

    def Get_NoiseFigure(self):
        NF = self.queryFloat('TRAC? TRACE1, NOIS')
        return NF

    def Get_YFactor(self):
        YFac = self.queryFloat('TRAC? TRACE1, YFAC')
        return YFac

    def Get_NoiseTemp(self):
        NTemp = self.queryFloat('TRAC? TRACE1, TEMP')
        return NTemp

    def Get_NF_CalCold(self):
        NCalCold = self.queryFloat('TRAC? TRACE1, CPC')
        return NCalCold

    def Get_NF_CalHot(self):
        NCalHot = self.queryFloat('TRAC? TRACE1, CPH')
        return NCalHot

    def Get_NF_PHot(self):
        NPHot = self.queryFloat('TRAC? TRACE1, PHOT')
        return NPHot

    def Get_NF_PCold(self):
        NPCold = self.queryFloat('TRAC? TRACE1, PCOL')
        return NPCold

    def System_Preset(self):
        self.write('SYST:PRES:CHAN')

###############################################################################
### Run if Main
###############################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSW = VSA()
    FSW.jav_Open("192.168.1.109")
    FSW.Init_Noise()
    FSW.jav_ClrErr()
    del FSW



"""Phase Noise Analyzer Common Functions"""
from rssd.yaVISA import jaVisa              # pylint: disable=E0611,E0401

class PNA(jaVisa):
    """ Rohde & Schwarz Vector Signal Analyzer Object """
    def __init__(self):
        super(PNA, self).__init__()
        self.Model = "FSWP"

    ############################################################################
    ### FSW Get
    ############################################################################
    def Get_AttnMech(self):
        out = self.queryInt('INP:ATT?')
        return out

    def Get_Channels(self):                                                     #done
        ChList = self.query('INST:LIST?').split(',')
        return(ChList)

    def Get_Freq(self):                                                         #done
        rdStr = self.queryFloat(':SENS:FREQ:CENT?')
        return rdStr

    def Get_FreqLock(self):                                                     #done
        rdstr = self.query(':STAT:QUES:PNO:COND?')
        return rdstr

    def Get_IFOvld(self):
        self.Set_InitImm()
        rdStr = self.query("STAT:QUES:POW:COND?").strip()
        return rdStr

    def Get_Harm_Values(self):
        rdStr = 'TBD'
        return rdStr

    def Get_Mkr_Freq(self,iNum=1,iWind=1):
        MkrFreq = self.queryFloat(':CALC%d:MARK%d:X?'%(iWind,iNum))
        return float(MkrFreq)

    def Get_Mkr_XY(self,iNum=1,iWind=1):
        ValX = self.query(':CALC%d:MARK%d:X?'%(iWind,iNum)).strip()
        ValY = self.query(':CALC%d:MARK%d:Y?'%(iWind,iNum)).strip()
        return [ValX, ValY]

    def Get_Mkr_Y(self,iNum=1,iWind=1):
        ValY = self.queryFloat(':CALC%d:MARK%d:Y?'%(iWind,iNum))
        return ValY

    def Get_Ovld_Stat(self):
        self.Set_InitImm()
        Read = self.queryInt('STAT:QUES:POW:COND?')
        RF_Ovld = Read & 1      # pylint: disable=W0612
        RF_Udld = Read & 2      # pylint: disable=W0612
        IF_Ovld = Read & 4      # pylint: disable=W0612
        return Read

    def Get_Power(self):
        rdStr = self.query('POW:RLEV?')
        return rdStr

    def Get_PN_Decade(self):
        """Phase Noise per decade (Spot Noise)"""
        rdStr = self.query('CALC:SNO:DEC:Y?')
        return rdStr

    def Get_PN_Int(self):
        """Integrated Phase Noise
        :FETC<n>:RANG<j>:PNO<t>:IPN?
        <n>   : Window
        <j>   : Integration Range
        <t>   : Trace """
        rdStr = self.query(':FETC:RANG:PNO:IPN?')
        return rdStr

    def Get_RefLevel(self):
        RefLvl = self.queryFloat('DISP:TRAC:Y:RLEV?')
        return RefLvl

    def Get_Screenshot(self,file='screenshot'):
        ### File will be in FSW's C:\R_S\Instr\User
        self.write(f'MMEM:NAME "C:\\R_S\\INSTR\\USER\\{file}"')
        self.write('HCOP:CONT WIND')     #Print Displayed Windo
        self.write('HCOP:CMAP:DEF4')     #Screeen Colors
        self.write('HCOP:DEST "MMEM"')   #Send Data to file
        self.write('HCOP:DEV:LANG JPG')  #Save JPG
        self.write('HCOP:IMM')           #Create File

    def Get_SweepPoints(self):
        rdStr = self.queryInt(':SENS:SWE:POIN?')            #Number of trace points
        return rdStr

    def Get_SweepTime(self):
        rdStr = self.queryFloat('SENS:SWE:TIME?')           #Sweep/Capture Time
        return rdStr

    def Get_SweepType(self):
        #AUTO | SWE | FFT
        rdStr = self.query(':SENS:SWE:TYPE?')
        return rdStr

    def Get_SweepParams(self):
        # ,SwpTime,SwpPts,SwpType,SwpOpt,
        Opt = self.Get_SweepOpt()
        Pts = self.Get_SweepPoints()
        Tim = self.Get_SweepTime()
        Typ = self.Get_SweepType()
        return f'{Tim},{Pts},{Typ},{Opt}'

    def Get_Trace_Data(self,trace=1):
        self.write('FORM ASCII ')
        DataY = self.query('TRAC%d:DATA? TRACE1'%trace)
        DataX = self.query('TRAC%d:DATA:X? TRACE1'%trace)
        return [DataX.split(','),DataY.split(',')]

    ###########################################################################
    ### Measurement Init
    ###########################################################################
    def Init_Harm(self):
        self.Set_Channel("Spectrum")
        self.write('CALC:MARK:FUNC:HARM ON')

    def Init_PhaseNoise(self):
        self.Set_Channel("Spectrum")
        self.write('CALC:MARK:FUNC:HARM ON')

    def Init_Spectral(self):
        self.Set_Channel("Spectrum")

    ###########################################################################
    ### Set Methods
    ###########################################################################
    def Set_AttnMech(self,fMAttn):
        #self.write('INP:EATT:STAT OFF')
        self.write('INP:ATT %.0f'%fMAttn)

    def Set_AttnAuto(self):
        self.write(':INP:ATT:AUTO ON')

    def Set_Autolevel(self):
        self.query(':SENS:ADJ:LEV;*OPC?')

    def Set_Autolevel_Proto(self,sState):
    ### Used by WLAN; K96.  Please use ADJ:LEV;
        self.write('CONF:POW:AUTO %s;*WAI'%sState)          #ON|OFF|1|0

    def Set_Channel(self,Chan,sName=""):
        """ SAN, IQ, NR5G, LTE, WLAN, PNOISE, NOISE, SPUR, ADEM, DDEM, V5GT, AMPL """
        if sName == "":
            sName = Chan
        ChList = self.query('INST:LIST?').split(',')
        #print("Chan:%s in %s"%(Chan,ChList))
        if ("'" + sName + "'") in ChList:
            pass
        else:
            self.query(":INST:CRE %s,'%s';*OPC?"%(Chan,sName))
        self.query(":INST:SEL '%s';*OPC?"%sName)

    def Set_DisplayUpdate(self,state):
        self.write('SYST:DISP:UPD %s'%state)                #Display Update State

    def Set_Freq(self,fFreq):
        self.write(':SENS:FREQ:CENT %.0f HZ'%fFreq)

    def Set_FreqStart(self,fFreq):                          #done
        self.write(':SENS:FREQ:STAR %f'%fFreq)

    def Set_FreqStep(self,fFreq):
        self.write(':SENS:FREQ:STEP %f'%fFreq)

    def Set_FreqStop(self,fFreq):                           #done
        self.write(':SENS:FREQ:STOP %f'%fFreq)

    def Set_Harm_num(self, num):
        self.write(f':CALC1:MARK1:FUNC:HARM:NHAR {num}')

    def Set_Harm_adjust(self):
        """ Adjusts Ref Lvl, Attn, SwpTime """
        self.write(':CALC1:MARK1:FUNC:HARM:PRE')

    def Set_InitImm(self):
        self.query('INIT:IMM;*OPC?')

    def Set_Input(self,sType):
        self.write('INP:SEL %s'%sType)                      #RF|AIQ|DIQ|FILE

    def Set_Mkr_AllOff(self,iWind=1):
        self.write(':CALC%d:MARK:AOFF'%(iWind))

    def Set_Mkr_Freq(self,fFreq,iNum=1,iWind=1):
        self.write(':CALC%d:MARK%d:X %fHz'%(iWind,iNum,fFreq))

    def Set_Mkr_Next(self,iNum=1,iWind=1):
        self.write(':CALC%d:MARK%d:MAX:NEXT'%(iWind,iNum))

    def Set_Mkr_On(self,iNum,iWind=1):
        self.write(':CALC%d:MARK%d ON'%(iWind,iNum))

    def Set_Mkr_Peak(self,iNum=1,iWind=1):
        self.write(':CALC%d:MARK%d:MAX:PEAK'%(iWind,iNum))

    def Set_Preamp(self,sState):
        #ON|OFF|1|0
        self.write('INP:GAIN:STAT %s;*WAI'%sState)

    def Set_PwrThreshold(self,dBm):
        self.write(f'SENS:ADJ:CONF:LEV:THR {dBm}')

    def Set_RefLevel(self,fReflevel):
        self.write('DISP:WIND:TRAC:Y:SCAL:RLEV %fdBm'%fReflevel)

    def Set_ResBW(self,fFreq):
        if fFreq == 0:
            self.write(':SENS:BAND:RES:AUTO ON')
        else:
            self.write(':SENS:BAND:RES %f'%fFreq)

    def Set_SamplingRate(self,fFreq):
        self.write('TRAC:IQ:SRAT %f'%fFreq)

    def Set_Span(self,fFreq):
        self.write('SENS:FREQ:SPAN %f'%fFreq)

    def Set_SweepType(self,sType):
        #AUTO | SWE | FFT
        self.write(f':SENS:SWE:TYPE {sType}')

    def Set_SweepOpt(self,sOpt):
        #AUTO | SPEed | DYN
        self.write(f':SENS:SWE:OPT {sOpt}')

    def Set_SweepCont(self,iON):
        ''' 0 1 '''
        if iON == 1:
            self.write('INIT:CONT ON')                      #Continuous Sweep
        else:
            self.write('INIT:CONT OFF')                     #Single Sweep

    def Set_SweepPoints(self,iNum):
        self.write(':SENS:SWE:POIN %f'%iNum)                #Number of trace points

    def Set_Trace_Avg(self,sType,trace=1):
        """LIN VID POW"""
        self.write('DISP:TRAC%d:MODE AVER'%trace)
        self.write('SENS:DET1:FUNC AVER')
        self.write('SENS:AVER:TYPE %s'%sType)               #LIN|VID

    def Set_Trace_AvgCount(self,iAvg):
        self.write('SENS:SWE:COUN %d'%(iAvg))

    def Set_Trace_AvgOff(self,trace=1):
        self.write('DISP:TRAC%d:MODE WRIT'%(trace))

    def Set_Trace_Detector(self,sDetect,iWind=1):
        """APE; NEG; POS; QPE; SAMP; RMS; CAV; CRMS"""
        self.write('SENS:WIND%d:DET %s'%(iWind,sDetect))

    def Set_VidBW(self,fFreq):
        if fFreq == 0:
            self.write(':SENS:BAND:VID:AUTO ON')
        else:
            self.write(':SENS:BAND:VID %f'%fFreq)

    def Set_Xcorr(self,iNum):                               #done
        self.write(f':SENS:SWE:XFAC {iNum}')

    def Set_XcorrOpt(self,State):                           #done
        """1 0"""
        if State in (1, '1', 'ON'):
            self.write(f':SENS:SWE:XOPT:STAT ON')
        else:
            self.write(f':SENS:SWE:XOPT:STAT OFF')

###############################################################################
### Run if Main
###############################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    FSWP = PNA().jav_Open("192.168.1.108")
    print(FSWP.Get_Freq())
    print(FSWP.Get_PN_Int())
    FSWP.jav_ClrErr()
    del FSWP



"""FSW/SMW LTE Demo"""
#pylint: disable=E0611,E0401
from rssd.VSG.LTE_K55    import VSG
from rssd.VSA.LTE_K100   import VSA

class VST(object):
    def __init__(self):
        self.Freq      = 2.5e9
        self.SWM_Out  = 0
        self.LTE_Dir    = 'DL'
        self.LTE_ChBW   = 5        #MHz
        self.LTE_RB     = 66        #RB
        self.LTE_RBO    = 0         #RB Offset
        self.LTE_Mod    = 'QAM64'   #QPSK; QAM16; QAM64; QAM256
        self.LTE_CC     = 1

    def Get_LTE_All(self):
        odata =  [[] for i in range(3)]
        odata[0].append("[[Parameter]]  ")
        odata[0].append("Direction      ")
        odata[0].append("Duplex         ")
        odata[0].append("Ch BW          ")
        odata[0].append("===User/BWP====")
        odata[0].append("RB             ")
        odata[0].append("RBoff          ")
        odata[0].append("Mod            ")

        try:
            odata[1].append("[-SMW-]")
            odata[1].append(self.SMW.Get_LTE_Direction())
            odata[1].append(self.SMW.Get_LTE_Duplex())
            odata[1].append(self.SMW.Get_LTE_ChBW())
            odata[1].append("=User=")
            odata[1].append(self.SMW.Get_LTE_ResBlock())
            odata[1].append(self.SMW.Get_LTE_ResBlockOffset())
            odata[1].append(self.SMW.Get_LTE_Modulation())
        except:
            pass

        try:
            self.FSW.Init_LTE()
            odata[2].append("[-FSW-]")
            odata[2].append(self.FSW.Get_LTE_Direction())
            odata[2].append(self.FSW.Get_LTE_Duplex())
            odata[2].append(self.FSW.Get_LTE_ChBW())
            odata[2].append("=User=")
            odata[2].append(self.FSW.Get_LTE_ResBlock())
            odata[2].append(self.FSW.Get_LTE_ResBlockOffset())
            odata[2].append(self.FSW.Get_LTE_Modulation())
        except:
            pass
        print('SMW/FSW Values: %d %d'%(len(odata[2]),len(odata[2])))

        return odata

    def Get_LTE_All_print(self):
        data = self.Get_LTE_All()
        for i in range(len(data[0])):
            try:
                print("%s\t%s\t%s"%(data[0][i],data[1][i],data[2][i]))
            except:
                try:
                    print("%s\t%s\t%s"%(data[0][i],data[1][i],'<notRead>'))
                except:
                    print("%s\t%s\t%s"%(data[0][i],'<notRead>',data[2][i]))

    def jav_Open(self,SMW_IP,FSW_IP,OFile=''):
        self.SMW = VSG().jav_Open(SMW_IP,OFile)  #Create SMW Object
        self.FSW = VSA().jav_Open(FSW_IP,OFile)  #Create FSW Object
        return self

    def jav_OpenTest(self,SMW_IP,FSW_IP):
        self.SMW = VSG().jav_OpenTest(SMW_IP)  #Create SMW Object
        self.FSW = VSA().jav_OpenTest(FSW_IP)  #Create FSW Object
        return self

    def jav_Close(self):
        self.SMW.jav_Close()
        self.FSW.jav_Close()

    def jav_Clear(self):
        self.SMW.jav_Clear()
        self.FSW.jav_Clear()

    def Set_LTE_All(self):
        try:
            ### SMW Settings
            self.SMW.Set_Freq(self.Freq)
            self.SMW.Set_LTE_BBState('OFF')
            self.SMW.Set_LTE_Direction(self.LTE_Dir)
            self.SMW.Set_LTE_ChBW(self.LTE_ChBW)
            self.SMW.Set_LTE_ResBlock(self.LTE_RB)
            self.SMW.Set_LTE_ResBlockOffset(self.LTE_RBO)
            self.SMW.Set_LTE_Modulation(self.LTE_Mod)
            self.SMW.Set_LTE_BBState('ON')
            self.SMW.Set_RFState('ON')                          #Turn RF Output on
            self.SMW.Set_RFPwr(self.SWM_Out)                    #Output Power
        except:
            print("NR5G_SetSettings: SMW Error")

        try:
            ### FSW Setting
            self.FSW.Init_LTE()
            self.FSW.Set_Freq(self.Freq)
            self.FSW.Set_LTE_Direction(self.LTE_Dir)
            self.FSW.Set_LTE_ChBW(self.LTE_ChBW)
            self.FSW.Set_LTE_ResBlock(self.LTE_RB)
            self.FSW.Set_LTE_ResBlockOffset(self.LTE_RBO)
            self.FSW.Set_LTE_Modulation(self.LTE_Mod)
            self.FSW.Set_SweepCont(1)
            self.FSW.Set_InitImm()
        except:
            print("NR5G_SetSettings: FSW Error")
        return 0


##########################################################
### Instrument Settings
##########################################################
if __name__ == "__main__":
    LTE = VST().jav_Open('192.168.1.114','192.168.1.109')
    LTE.LTE_ChBW = 10
    LTE.Set_LTE_All()
    LTE.Get_LTE_All_print()
    LTE.jav_Close()



###############################################################################
### Code Overhead: Import and create objects
###############################################################################
from rssd.VSG.NR5G_K144    import VSG

SMW = VSG().jav_Open('10.0.0.10')  #Create SMW Object

###############################################################################
### User Entry
###############################################################################
SlotDir     = 'DDDSU'       #Frame Pattern
Entities    = [1,2,3,4]
RVPatt      = [0,2,3,1]
SlotNum     = 8             #Num of Slots per Subframe
SubFNum     = 10            #Num of SubFrame per Frame
currSlot    = 0
RVIi        = 0
FrameInfo   = []
numAlloc    = [0]*SubFNum   #Num Alloc per SubFrame

for SubF in range(SubFNum):                             # Loop through all slots in Frame individually 
    numAlloc[SubF] = 0                                  # Start Each Frame w/ 0 allocations
    FrameInfo.append([])                                # Blank Slot Definition
    for Slot in range(SlotNum):
        UDdir = currSlot % len(SlotDir)                 # Retrieve Uplink/Downlink Direction from UDdir
        if 'U' in SlotDir[UDdir]:                       # If this slot is U
            numAlloc[SubF] += 1                         #   - Add Allocation
            RVI = RVPatt[RVIi%len(RVPatt)]              #   - Identify next RVI in RVPatt
            FrameInfo[SubF].append([SubF,Slot,RVI])     #   - Add SubFrame; Slot# and RVI into our Frame Definition
            print(f'SubF:{SubF} Slot:{Slot} RVI:{RVI}')  #   - Print for verification.
            RVIi += 1
        currSlot += 1

for ent in Entities:
    SMW.write(f'ENT{ent}:SOUR1:BB:NR5G:STAT 0')         # Baseband off
    for SubF in range(SubFNum):
        SMW.write(f':ENT{ent}:SOUR1:BB:NR5G:SCH:CELL0:SUBF{SubF}:USER0:BWP0:NALL {numAlloc[SubF]}')
        for alloc in range(numAlloc[SubF]):
            RVI = FrameInfo[SubF][alloc][2]
            SMW.write(f':ENT{ent}:SOUR1:BB:NR5G:SCH:CELL0:SUBF{SubF}:USER0:BWP0:ALL{alloc}:SLOT {FrameInfo[SubF][alloc][1]}')
            SMW.write(f':ENT{ent}:SOUR1:BB:NR5G:SCH:CELL0:SUBF{SubF}:USER0:BWP0:ALL{alloc}:REP OFF')
            SMW.write(f':ENT{ent}:SOUR1:BB:NR5G:SCH:CELL0:SUBF{SubF}:USER0:BWP0:ALL{alloc}:PUSCh:CCOD:RVIN {RVI}')
    SMW.jav_Wait(f'ENT{ent}:SOUR1:BB:NR5G:STAT 1')      # Baseband On & Calculate



"""Test FileIO.py
"""
import os
import unittest
from rssd.FileIO      import FileIO             # pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                            #Run before each test
        self.FileIO = FileIO()
        self.FileIO.init("FileIO.csv")
        self.FileIO.debug = 0

    def tearDown(self):                         #Run after each test
        self.FileIO.Outfile.close()

###############################################################################
### </Test>
###############################################################################
    def test_setFilename(self):
        self.FileIO.set_filename(".system")

    def test_makeFile(self):
        newName = 'makeFile'
        self.FileIO.makeFile(newName)
        self.assertNotEqual(self.FileIO.sFName.find(newName),-1)

    def test_readcsv(self):
        self.FileIO.write('testreadcsv,spam,ham,eggs')
        data = self.FileIO.readcsv()            #Read as 2D Table
        size = len(data)
        self.assertEqual(data[size-1][3],'ham')

    def test_read(self):
        self.FileIO.write_raw('testread,spam,ham,eggs')
        data = self.FileIO.read()               #Read entire file
        size = len(data)
        self.assertEqual(data[size-1].strip(),'testread,spam,ham,eggs')
        # self.assertTrue('FOO'.isupper())
        # self.assertFalse('Foo'.isupper())

    def test_readdict(self):
        BaseDir  = os.path.dirname(os.path.realpath(__file__))
        self.FileIO.set_filename(os.path.join(BaseDir,'.system'))
        rdDict = self.FileIO.readdict()
        rdDict['VSG']           #pylint: disable=W0104

    def test_write(self):
        self.FileIO.write("test_write,write")           #Append Date
        self.FileIO.write_raw("test_write,write_raw")   #No Date
        # self.assertTrue(1)                  #FAIL

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':                              # pragma: no cover
    # unittest.main()     #Run w/o test names
    BaseDir  = os.path.dirname(os.path.realpath(__file__))
    fily = FileIO()
    fily.set_filename(f'{BaseDir}\\.system')
    d = fily.readdict()
    
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
### Record SCPI into file
###############################################################################
# from rssd.VSG.NR5G_K144     import VSG                              #pylint: disable=E0611,E0401
# from rssd.VSA.NR5G_K144     import VSA                              #pylint: disable=E0611,E0401
from rssd.RCT.NR5G_KM601    import RCT                              #pylint: disable=E0611,E0401
# from rssd.FileIO            import FileIO                           #pylint: disable=E0611,E0401

CMP  = RCT().jav_Open('192.168.1.160')                              #Create instrument Object
CMP.jav_logscpi()                                                   #Log SCPI to file

class dataClass():
    def __init__(self):
        self.Direction      = 'UL'
        self.CellID         = 1
        # self.FreqRng      = 'HIGH'
        self.ChBW           = 100
        self.TF             = 'ON'
        self.SubSp          = 120
        self.RB             = 60
        self.RBO            = 0
        self.Ch_RB          = 60
        self.Ch_RBO         = 0
        self.Mod            = 'QPSK'
        self.Rx             = ''
        self.pwr            = -100

###############################################################################
### SCPI Code we want to log
###############################################################################
NR5G = dataClass()
CMP.Init_5GNR()
CMP.Set_5GNR_Path('P1.RRH.RF1')
CMP.Set_Meas_Port('P1.RRH.RF1')
CMP.Set_5GNR_BWP_Frame_Periodicity(2)
CMP.Set_5GNR_BWP_Frame_SlotConfig(0,0,8,0)
CMP.Set_5GNR_ChannelBW(100)
CMP.Set_5GNR_CellID(NR5G.CellID)
CMP.Set_5GNR_BWP_Ch_DMRS_1stDMRSSym(2)
# CMP.Set_5GNR_NumBWP()
CMP.Set_5GNR_BWP_ResBlock(NR5G.RB, NR5G.RBO)
CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP:PUSC:DMTA BWP0, 1, 2, 1')                 #Config; AddPos; MaxLength
CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:BWP:PUSC:DMTB BWP0, 1, 2, 1')                 #Config; AddPos; MaxLength
CMP.Set_5GNR_TransPrecoding(NR5G.TF)
CMP.Set_5GNR_PUSCH(NR5G.Ch_RB, NR5G.Ch_RBO, NR5G.Mod)
CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:ALL{CMP.alloc}:PUSC:ADD 1, 2, 3, 0')          #Len; CDM; Pwr; Ant
CMP.write(f'CONF:NRMM:MEAS:CC{CMP.cc}:ALL{CMP.alloc}:PUSC:SGEN CID, 0, 0')          #SeqType; DMRSID; N_SCID
CMP.Set_5GNR_EVM_MeasOnExcept('ON')
CMP.Set_5GNR_EVM_AvgCount(20)
CMP.Set_5GNR_Trigger_Source('Free Run (Fast Sync)')



from __future__ import print_function
import unittest

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #Run before each test
        print("",end="")

    def tearDown(self):                             #Run after each test
        pass

###############################################################################
### <Test>
###############################################################################
    def test_FileIO(self):
        from rssd.FileIO import FileIO              #pylint:disable=E0611,E0401
        self.FileIO = FileIO()

    def test_NRP_Common(self):
        from rssd.NRP.Common import PMr             #pylint:disable=E0611,E0401
        self.NRP = PMr()
        self.assertEqual(self.NRP.Model,"NRP")

    def test_NRQ_Common(self):
        from rssd.NRQ.Common import NRQ             #pylint:disable=E0611,E0401
        self.NRQ = NRQ()
        self.assertEqual(self.NRQ.Model,"NRQ")

    def test_OSP_Common(self):
        from rssd.OSP.Common import OSP             #pylint:disable=E0611,E0401
        self.OSP = OSP()
        self.assertEqual(self.OSP.Model,"OSP1x0")

    def test_OTA_Common(self):
        from rssd.OTA.Common import OTA             #pylint:disable=E0611,E0401
        self.OTA = OTA()
        self.assertEqual(self.OTA.Model,"OTA")

    def test_OTA_ATS1000(self):
        from rssd.OTA.ATS1000 import OTA            #pylint:disable=E0611,E0401
        self.OTA = OTA()
        self.assertEqual(self.OTA.Model,"ATS1000")

    def test_OTA_ATS1800(self):
        from rssd.OTA.ATS1800 import OTA            #pylint:disable=E0611,E0401
        self.OTA = OTA()
        self.assertEqual(self.OTA.Model,"ATS1800")

    def test_PNA_Common(self):
        from rssd.PNA.Common import PNA             #pylint:disable=E0611,E0401
        self.PNA = PNA()
        self.assertEqual(self.PNA.Model,"FSWP")

    def test_RCT_Common(self):
        from rssd.RCT.Common import RCT             #pylint:disable=E0611,E0401
        self.CMW = RCT()
        self.assertEqual(self.CMW.Model,"CMW-GPRF")

    def test_VNA_Common(self):
        from rssd.VNA.Common import VNA             #pylint:disable=E0611,E0401
        self.VNA = VNA()        
        self.assertEqual(self.VNA.Model,"VNA")

    def test_VSA_ADemod(self):
        from rssd.VSA.ADemod_K7 import VSA          #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_Common(self):
        from rssd.VSA.Common import VSA             #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_LTE(self):
        from rssd.VSA.LTE_K100 import VSA           #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_NoiseFigure(self):
        from rssd.VSA.NoiseFigure_K30 import VSA    #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_5GNR(self):
        from rssd.VSA.NR5G_K144 import VSA          #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_Transient(self):
        from rssd.VSA.Transient_K60 import VSA      #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_VectorDemod(self):
        from rssd.VSA.VSA_K70 import VSA            #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSA_WLAN(self):
        from rssd.VSA.WLAN_K91 import VSA           #pylint:disable=E0611,E0401
        self.FSW = VSA()
        self.assertEqual(self.FSW.Model,"FSW")

    def test_VSE_Common(self):
        from rssd.VSE.Common import VSE             #pylint:disable=E0611,E0401
        self.VSE = VSE()        
        self.assertEqual(self.VSE.Model,"VSE")

    def test_VSE_K96(self):
        from rssd.VSE.K96 import VSE                #pylint:disable=E0611,E0401
        self.VSE = VSE()
        self.assertEqual(self.VSE.Model,"VSE")

    def test_VSG_Common(self):
        from rssd.VSG.Common import VSG             #pylint:disable=E0611,E0401
        self.SMW = VSG()        
        self.assertEqual(self.SMW.Model,"SMW")

    def test_VSG_CDM(self):
        from rssd.VSG.CustomDigMod import VSG       #pylint:disable=E0611,E0401
        self.SMW = VSG()        
        self.assertEqual(self.SMW.Model,"SMW")

    def test_VSG_Fading(self):
        from rssd.VSG.Fading import VSG             #pylint:disable=E0611,E0401
        self.SMW = VSG()        
        self.assertEqual(self.SMW.Model,"SMW")

    def test_VSG_LTE_K55(self):
        from rssd.VSG.LTE_K55 import VSG            #pylint:disable=E0611,E0401
        self.SMW = VSG()        
        self.assertEqual(self.SMW.Model,"SMW")

    def test_VSG_5GNR(self):
        from rssd.VSG.NR5G_K144 import VSG          #pylint:disable=E0611,E0401
        self.SMW = VSG()
        self.assertEqual(self.SMW.Model,"SMW")

    def test_VSG_WLAN_K54(self):
        from rssd.VSG.WLAN_K54 import VSG           #pylint:disable=E0611,E0401
        self.SMW = VSG()        
        self.assertEqual(self.SMW.Model,"SMW")

    def test_VST_Common(self):
        from rssd.VST.Common import VST             #pylint:disable=E0611,E0401
        self.VST = VST()
        self.assertEqual(self.VST.Freq,19e9)

    def test_VST_LTE(self):
        from rssd.VST.LTE import VST                #pylint:disable=E0611,E0401
        self.VST = VST()
        self.assertEqual(self.VST.LTE_CC,1)

    def test_VST_NR5G(self):
        from rssd.VST.NR5G_K144 import VST          #pylint:disable=E0611,E0401
        self.VST = VST()
        self.assertEqual(self.VST.NR_TF,'OFF')

    def test_VST_WLAN(self):
        from rssd.VST.WLAN import VST               #pylint:disable=E0611,E0401
        self.VST = VST()
        self.assertEqual(self.VST.WLAN_MCS,1)

    def test_yaVISA(self):
        from rssd.yaVISA import jaVisa              #pylint:disable=E0611,E0401
        self.K2 = jaVisa()

    def test_yaVISASocket(self):
        from rssd.yaVISA_socket import jaVisa       #pylint:disable=E0611,E0401
        self.K2 = jaVisa()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':                          # pragma: no cover
    # unittest.main()     #Run w/o test names
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



""" rssd.yaVISA.py driver test
"""
from __future__ import print_function
import unittest
from rssd.yaVISA import jaVisa

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #Run before each test
        self.instr = jaVisa().jav_OpenTest('192.168.1.108')
        self.instr.jav_ClrErr()
        print("",end="")
        self.instr.debug = 0

    def tearDown(self):                             #Run after each test
        try:
            self.instr.jav_Close()
        except:
            pass

###############################################################################
### <Test>
###############################################################################
    def test_yaVISA_Basic(self):
        # self.instr.write('')
        # self.instr.jav_write_raw('')
        self.instr.query('')
        # self.instr.jav_read_raw()

    def test_yaVISA_Clear(self):
        self.instr.jav_Clear()
        self.instr.jav_Error()

    def test_yaVISA_Delay(self):
        self.instr.delay(0.1)

    def test_yaVISA_IDN(self):
        self.instr.jav_IDN()
        self.instr.jav_Error()

    def test_yaVISA_LogSCPI(self):
        self.instr.jav_logscpi()
        # self.instr.f.close()

    def test_yaVISA_OPCWait(self):
        self.instr.jav_OPC_Wait('')

    def test_yaVISA_Query(self):
        self.instr.queryFloat('')
        self.instr.queryFloatArry('')
        self.instr.queryInt('')
        self.instr.queryIntArry('')

    def test_yaVISA_Raw(self):
        self.instr.jav_write_raw(b'1234')
        self.instr.jav_read_raw()

    def test_yaVISA_Reset(self):
        self.instr.jav_Reset()

    def test_yaVISA_Resources(self):
        self.instr.jav_reslist()

    def test_yaVISA_SCPIList(self):
        self.instr.jav_scpilist(['*IDN','*OPC?'])

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    # coverage run -a -m unittest -b -v test_SW_yaVISA_socket
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""FSW/SMW 5G NR Demo
"""
#pylint: disable=E0611,E0401
from rssd.VSG.Common import VSG
from rssd.VSA.Common import VSA

class VST(object):
    """ Rohde & Schwarz Vector Signal Transceiver Object """
    def __init__(self):
        self.Freq       = 19e9
        self.SMW        = ''
        self.FSW        = ''

    def jav_Open(self,SMW_IP,FSW_IP,OFile=''):
        self.SMW = VSG().jav_Open(SMW_IP,OFile)  #Create SMW Object
        self.FSW = VSA().jav_Open(FSW_IP,OFile)  #Create FSW Object
        return self

    def jav_OpenTest(self,SMW_IP,FSW_IP):
        self.SMW = VSG().jav_OpenTest(SMW_IP)  #Create SMW Object
        self.FSW = VSA().jav_OpenTest(FSW_IP)  #Create FSW Object
        return self

    def jav_Close(self):
        self.SMW.jav_Close()
        self.FSW.jav_Close()

    def jav_ClrErr(self):
        self.SMW.jav_ClrErr()
        self.FSW.jav_ClrErr()

    def Set_Freq(self,freq):
        self.SMW.Set_Freq(freq)
        self.FSW.Set_Freq(freq)

##########################################################
### Instrument Settings
##########################################################
if __name__ == "__main__":
    VST = VST().jav_Open('192.168.1.114','192.168.1.109')
    VST.Set_Freq(9e9)
    VST.jav_Close()



###############################################################################
### Rohde & Schwarz Driver Test
### Purpose: rssd.nrq_common.py driver test
### Author:  mclim
### Date:    2018.08.23
###              _   ___        __  _____         _   
###             | | | \ \      / / |_   _|__  ___| |_ 
###             | |_| |\ \ /\ / /    | |/ _ \/ __| __|
###             |  _  | \ V  V /     | |  __/\__ \ |_ 
###             |_| |_|  \_/\_/      |_|\___||___/\__|
###             Please connect instrument prior 2 test
###############################################################################
### User Entry
###############################################################################
host = '192.168.1.40'              #Get local machine name
#host = 'NRQ6-101507.local'                #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.NRQ.Common import NRQ

class TestGeneral(unittest.TestCase):
    def setUp(self):                      #run before each test
        self.NRQ6 = NRQ().jav_OpenTest(host)

    def tearDown(self):                         #Run after each test
        self.assertEqual(self.NRQ6.jav_Error()[0],'0')
        self.NRQ6.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_NRQ_Connect(self):
        if self.NRQ6.connected: self.assertEqual(self.NRQ6.Make,"ROHDE&SCHWARZ")

    def test_NRQ_Example(self):
        self.NRQ6.Set_DisplayUpdate('ON')
        self.NRQ6.Set_Freq(1e9)
        self.NRQ6.Set_Attn(10)
        self.NRQ6.Set_Attn(0)
        # self.NRQ6.Get_IQtoIQW()             #Output to file
        self.NRQ6.Get_IQ_Data()             #Output to varable.

    def test_NRQ_Freq(self):
        SetVal = 1e9
        self.NRQ6.Set_Freq(SetVal)
        GetVal = self.NRQ6.Get_Freq()
        if self.NRQ6.connected: self.assertEqual(SetVal,int(GetVal))

    def test_NRQ_Get(self):
        self.NRQ6.Get_Channels()

    def test_NRQ_IQ_2IQW(self):
        self.NRQ6.Set_DisplayUpdate('OFF')
        self.NRQ6.Get_IQtoIQW()

    def test_NRQ_IQ_RecLength(self):
        SetVal = 2468
        self.NRQ6.Set_IQ_RecLength(SetVal)
        GetVal = self.NRQ6.Get_IQ_RecLength()
        if self.NRQ6.connected: self.assertEqual(SetVal,int(GetVal))

    def test_NRQ_IQ_SamplingRate(self):
        SetVal = 12345678
        self.NRQ6.Set_IQ_SamplingRate(SetVal)
        GetVal = self.NRQ6.Get_IQ_SamplingRate()
        if self.NRQ6.connected: self.assertTrue(SetVal<int(GetVal))

    def test_NRQ_Power(self):
        GetVal = self.NRQ6.Get_Power()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
#coverage run -a -m unittest -b -v test_HW_NRQ_Common
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=4).run(suite)



"""NRQ Common Functions"""
from rssd.yaVISA import jaVisa

class NRQ(jaVisa):
    """ Rohde & Schwarz NRQ6 Object """
    def __init__(self):
        super(NRQ, self).__init__()
        self.Model = "NRQ"

    #####################################################################
    ### NRQ Display
    #####################################################################
    def Get_Channels(self):
        ChList = self.query('INST:LIST?').split(',')
        return ChList

    def Set_DisplayUpdate(self,state):
        """ON|OFF"""
        if state in (1,'1','ON'):
            self.write('SYST:DISP:UPD ON')                  #Display Update State
        elif state in (0,'0','OFF'):
            self.write('SYST:DISP:UPD OFF')                 #Display Update State

    #####################################################################
    ### NRQ Common Settings
    #####################################################################
    def Set_Freq(self,fFreq):
        self.write('SENS:FREQ %.0f'%fFreq)

    def Get_Freq(self):
        SRead = self.queryFloat(':SENS:FREQ?')
        return SRead

    def Set_Attn(self,fAttn):
        if fAttn == 0:
            self.write(':SENS:INP:ATT:AUTO ON')
        else:
            self.write(':SENS:INP:ATT:AUTO OFF')
            self.write(':SENS:INP:ATT %f'%fAttn)

    def Get_Power(self):
        self.write('UNIT:POW DBM')
        self.write('SENS:FUNC "POW:AVG"')
        self.write('INIT:CONT ON')
        fRead = self.queryFloat('FETCH?')
        return fRead

    #####################################################################
    ### NRQ IQ Commands
    #####################################################################
    def Get_IQ_SamplingRate(self):
        # SamplingRate = IQ_BW / 0.8
        iRead = self.queryFloat(':SENS:BAND:SRAT?')             #Sampling Rate
        return iRead

    def Set_IQ_SamplingRate(self,fFreq):
        # SamplingRate = IQ_BW / 0.8
        self.write('SENS:BAND:TYPE SRAT')
        self.write('SENS:BAND:VAR ON')
        self.write('SENS:BAND:SRAT %d'%fFreq)                   #Sampling Rate

    def Get_IQ_RecLength(self):
        iRead = self.queryInt('SENS:TRAC:IQ:RLEN?')
        return iRead

    def Set_IQ_RecLength(self,iLen):
        self.write('SENS:TRAC:IQ:RLEN %d'%iLen)

    def Get_IQ_Data(self):
        """ NRQ IQ Data --> Variable """
        self.write('SENS:FUNC “XTIM:VOLT:IQ”')
        self.write('SENS:BAND:TYPE SRAT')
        self.write('TRIG:SOUR IMM')
        self.write('SENS:TRAC:IQ:DATA:FORM IQPAIR')
        self.write('FORM:DATA REAL,32')
        self.write('INIT:IMM')
        rdStr = self.query('FETCH?')
        return rdStr

    def Get_IQtoIQW(self):
        """ NRQ IQ data --> IQW file to process in VSE """
        self.write("SENS:FUNC 'XTIM:VOLT:IQ'")
        self.write("SENS:TRAC:IQ:DATA:FORM IQPAIR")
        self.write("FORM:DATA REAL,32")
        #self.write("FORM:DATA ASCII")

        self.write("INIT:IMM")
        self.write("FETCH?")
        data = self.jav_read_raw()

        # Read num of digits to get for No of floats
        if self.Get_IQ_RecLength() < 125000000:
            digits = data[1]
            print(digits)
        else:                                                   # pragma: no cover
            digits = "10"

        # Don't need this but including for completeness
        # Reads total number of bytes that holds IQ data
        # i = 2
        # totalbytes = ""
        # while i <= int(digits)+1:
        #     totalbytes = totalbytes + data [i]
        #     i += 1

        iqfile = open('nrq.iqw', "wb")
        iqfile.write(data[2 + int(digits):])
        iqfile.close()

#####################################################################
### Run if Main
#####################################################################
if __name__ == "__main__":
    ### this won't be run when imported
    NRQ6 = NRQ()
#    NRQ6.jav_Open("NRQ6-101507.local")
    NRQ6.jav_Open("192.168.1.40")
    NRQ6.Get_IQtoIQW()
    NRQ6.jav_ClrErr()



###############################################################################
### Rohde & Schwarz Automation for demonstration use.
### Title  : Timing SCPI Commands Example
### Author : mclim
### Date   : 2018.05.24
###
###############################################################################
### User Entry
###############################################################################
VSA_IP  = '192.168.1.109'
VSG_IP  = '192.168.1.114'
MeasTim = 100e-6
Freq    = 9e9
ChBW    = 95e6
ChSpace = 100e6
Avg     = 0

### Loops
Repeat  = 1
PwrSweep = 59

meth = {0:'VSA.Set_Autolevel()',
        1:'VSA.Set_Mkr_BandSetRef()'
        }

###############################################################################
### Code Overhead
###############################################################################
import timeit
from rssd.VSA.Common        import VSA              #pylint: disable=E0611,E0401
from rssd.yaVISA_socket     import jaVisa           #pylint: disable=E0611,E0401
from rssd.FileIO            import FileIO           #pylint: disable=E0611,E0401

OFile = FileIO().makeFile(__file__)
VSA = VSA().jav_Open(VSA_IP,OFile)                  #Create VSA Object
VSG = jaVisa().jav_Open(VSG_IP,OFile)               #Create Object
###############################################################################
### Code Start
###############################################################################
#VSA.jav_Reset()
VSA.Init_IQ()                                       #FSW ACLR Channel
VSA.Set_Freq(Freq)
VSA.Set_IQ_ACLR(ChBW, ChSpace)

#VSA.Set_DisplayUpdate("OFF")
VSA.Set_Param_Couple_All()
VSA.Set_SweepTime(MeasTim)
VSA.Set_Trace_Avg('LIN')
VSA.Set_Trace_AvgCount(Avg)
VSA.Set_Trace_Detector('RMS')
VSA.Set_In_YIG('OFF')
# VSA.Set_Trig1_Source('Ext')

###############################################################################
### Measure Time
###############################################################################
#sDate = datetime.now().strftime("%y%m%d-%H:%M:%S.%f") #Date String
LoopParam   = 'Iter,ALMeth,Pwr'
TimeParam   = 'AlTime,MeasTime,TotalTIme'
SwpParam    = VSA.Get_Params_Sweep(1)
AmpParam    = VSA.Get_Params_Amp(1)
TrcParam    = VSA.Get_Params_Trace(1)
SysParam    = VSA.Get_Params_System(1)
MeasData    = 'TxPwr,Adj-,Adj+,Alt-,Alt+'
OFile.write(f'{LoopParam},{TimeParam},{AmpParam},{SwpParam},{TrcParam},{SysParam},{MeasData}')

# OFile.write('Model    ,Iter,Freq,RBW,SwpTime,SMWPwr,ALType,ALTime,TotTime,Attn,PreAmp,RefLvl,SwpTime,SwpPts,SwpType,SwpOpt,TxPwr,Adj-,Adj+,Alt-,Alt+,ChSpace')
tick0 = timeit.default_timer()
for i in range(Repeat):
    for autoMeth in meth:
        for VSApwr in range(PwrSweep):
            ### <\thing we are timing>
            VSG.write(f':POW:AMPL {-50 + VSApwr}dbm')               # VSG Power
            tick = timeit.default_timer()

            ### <AUTOLEVEL> ###
            eval(autoMeth)                                          #pylint: disable=W0123
            tockA = timeit.default_timer()
            ### <AUTOLEVEL> ###

            VSA.write(':INIT:CONT OFF')                             # Single Sweep
            VSA.query(':INIT:IMM;*OPC?')                            # Take Sweep
            ACLR = VSA.Get_Mkr_BandACLR()
            ### <\thing we are timing>

            tockB = timeit.default_timer()
            ALTime = tockA - tick
            TotTime = tockB - tick
            TestTime = TotTime - ALTime

            LoopParam   = f'{i},{autoMeth},{VSApwr:5.2f}'
            SwpParam    = VSA.Get_Params_Sweep()
            AmpParam    = VSA.Get_Params_Amp()
            TrcParam    = VSA.Get_Params_Trace()
            SysParam    = VSA.Get_Params_System()
            TotTime     = f'{ALTime:2,.6f},{TestTime:2,.6f},{TotTime:2,.6f}'
            MeasData    = ACLR
            OutStr      = f'{LoopParam},{TotTime},{AmpParam},{SwpParam},{TrcParam},{SysParam},{MeasData}'

            # OutStr = f'{VSA.Model},{i},{Freq},{RBW},{MeasTim},{-50+pwr},{meth[autoMeth]},{ALTime.seconds:3d}.{ALTime.microseconds:06d},{TotTime.seconds:3d}.{TotTime.microseconds:06d},{AmpSet},{SwpParam},{ACLR},{ChSpace}'
            OFile.write(OutStr)
SuiteTime = timeit.default_timer() - tick0
print(f'Total Test time : {SuiteTime:2,.6f}')
print(f'Time/Measurement: {SuiteTime/(Repeat*PwrSweep):2,.6f}')

###############################################################################
### Cleanup Automation
###############################################################################
OFile.write("\n")
VSA.jav_Close()



"""rssd.PNA.common test
"""
host = '192.168.1.109'                              #Get local machine name

###############################################################################
### Code Start
###############################################################################
import unittest
from rssd.PNA.Common        import PNA              #pylint: disable=E0611,E0401

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #run before each test
        self.FSWP = PNA().jav_OpenTest(host)

    def tearDown(self):                             #Run after each test
        self.assertEqual(self.FSWP.jav_Error()[0],'0')
        self.FSWP.jav_Close()

###############################################################################
### <Test>
###############################################################################
    def test_FSWP_Freq(self):
        self.FSWP.Init_Spectral()
        self.FSWP.Set_FreqStart(10e6)
        self.FSWP.Set_FreqStop(100e6)
        self.FSWP.Set_Freq(100e6)
        rdStr = self.FSWP.Get_Freq()
        #var = input("Please enter something: ")
        if self.FSWP.connected: self.assertEqual(100e6,rdStr)                    # Valuecompare

    def test_FSWP_Get(self):
        self.FSWP.Init_PhaseNoise()
        self.FSWP.Get_AttnMech()
        self.FSWP.Get_Channels()
        self.FSWP.Get_RefLevel()
        self.FSWP.Get_SweepPoints()
        self.FSWP.Get_SweepTime()
        self.FSWP.Get_SweepType()

    def test_FSWP_GetScreenshot(self):
        getVal = self.FSWP.Get_Screenshot()

    def test_FSWP_GetTrace(self):
        getVal = self.FSWP.Get_Trace_Data()

    def test_FSWP_Harmonics(self):
        self.FSWP.Init_Harm()
        self.FSWP.Set_Harm_num(3)
        self.FSWP.Set_Harm_adjust()
        getVal = self.FSWP.Get_Harm_Values()

    def test_FSWP_Input(self):
        self.FSWP.Set_Freq(1e9)
        self.FSWP.Set_Input('RF')
        # self.FSWP.Set_In_YIG('ON')
        # self.FSWP.Set_In_HPFilter('ON')

    def test_FSWP_Marker(self):
        self.FSWP.Init_Spectral()
        self.FSWP.Set_Mkr_AllOff()
        self.FSWP.Set_Mkr_Peak()
        self.FSWP.Set_Mkr_On(2)
        self.FSWP.Set_Mkr_Next()
        getVal = self.FSWP.Get_Mkr_Freq()
        getVal = self.FSWP.Get_Mkr_XY()
        getVal = self.FSWP.Get_Mkr_Y()

    def test_FSWP_PhaseNoise(self):
        self.FSWP.Init_PhaseNoise()
        self.FSWP.Set_SweepCont(0)          # Single Sweep
        self.FSWP.Set_PwrThreshold(-10)
        self.FSWP.Set_Xcorr(100)
        self.FSWP.Set_XcorrOpt(1)
        getVal = self.FSWP.Get_FreqLock()
        getVal = self.FSWP.Get_Freq()
        getVal = self.FSWP.Get_Power()
        getVal = self.FSWP.Get_PN_Decade()
        getVal = self.FSWP.Get_PN_Int()

    def test_FSWP_Spectral_Settings(self):
        self.FSWP.Init_Spectral()
        self.FSWP.Set_SweepCont(1)          # Contiuous Sweep
        self.FSWP.Set_Freq(1e6)
        self.FSWP.Set_RefLevel(10)
        self.FSWP.Set_ResBW(1e6)
        self.FSWP.Set_ResBW(0)              # Auto
        self.FSWP.Set_VidBW(1e6)
        self.FSWP.Set_Span(100e6)
        self.FSWP.Set_AttnMech(10)
        self.FSWP.Set_AttnAuto()
        self.FSWP.Set_Autolevel()
        self.FSWP.Get_IFOvld()
        self.FSWP.Set_DisplayUpdate('ON')

    def test_FSWP_Sys_Overload(self):
        getVal = self.FSWP.Get_Ovld_Stat()

    def test_FSWP_Trace(self):
        self.FSWP.Init_PhaseNoise()
        self.FSWP.Set_Trace_Avg('POW')
        self.FSWP.Set_Trace_Detector('RMS')
        self.FSWP.Set_Trace_AvgCount(20)
        self.FSWP.Set_Trace_AvgOff()

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    #coverage run -a -m unittest discover -b -v -p test_HW_PNA_Common.py
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""5G NR FSW/SMW Carrier Aggregation Setup Example"""
###############################################################################
### User Entry
###############################################################################
SMW_IP      = '192.168.58.114'
FSW_IP      = '192.168.58.109'
FreqArry    = range(9500000000, 12900000000, 50000000)
pwrArry     = [-5]                              #Power Array
NR_Dir      = 'UL'
waveparam   = [[100,120,66,'QPSK']]                #ChBW, SubSp, RB, Mod
NumCC       = 14
CCSpace     = 99.96e6
CCStart     = (1 - NumCC) * (CCSpace/2)
numMeas     = 1
DFT_S_OFDM  = 'ON'

###############################################################################
### Code Overhead: Import and create objects
###############################################################################
import timeit
from rssd.FileIO                import FileIO       #pylint: disable=E0611,E0401
from rssd.VST.NR5G_K144         import VST          #pylint: disable=E0611,E0401
OFile = FileIO().makeFile(__file__)

###############################################################################
### Code Start
###############################################################################
NR5G = VST().jav_Open(SMW_IP,FSW_IP,OFile)
NR5G.NR_TF      = DFT_S_OFDM
NR5G.NR_Dir     = NR_Dir
NR5G.Freq       = FreqArry[0]

###############################################################################
### Measure Time
###############################################################################
LoopParam   = 'Iter,Model,SMW_Fre,CCFreq_GHz,SMW_Pwr,CC,NumCC'
WaveParam   = 'ChBW,SubSp,RB,Mod,TF'
SwpParam    = NR5G.FSW.Get_Params_Sweep(1)
AttnParam   = NR5G.FSW.Get_Params_Amp(1)
EVMParam    = NR5G.FSW.Get_5GNR_Params_EVM(1)
TimeParam   = 'AlTime,MeasTime,TotalTIme'
Header      = f'{LoopParam},{WaveParam},{AttnParam},{EVMParam},{TimeParam}'
OFile.write(Header)

NR5G.FSW.Init_5GNR()
NR5G.FSW.Set_5GNR_EVMUnit('DB')
NR5G.FSW.Set_Trig1_Source('IMM')
NR5G.FSW.Set_5GNR_Direction(NR_Dir)
NR5G.FSW.Set_SweepCont(0)
NR5G.FSW.Set_5GNR_FreqRange(2)
NR5G.FSW.Set_5GNR_CC_Num(NumCC)
NR5G.FSW.Set_5GNR_SubFrameCount(10)
NR5G.FSW.Set_SweepTime(10.1e-3)
NR5G.FSW.Set_5GNR_Result_View('ALL')

for i in range(numMeas):                                            #LOOP: Measurements
    for param in waveparam:                                         #LOOP: Waveform Parameters
        NR5G.NR_ChBW    = param[0]
        NR5G.NR_SubSp   = param[1]
        NR5G.NR_RB      = param[2]
        NR5G.NR_Mod     = param[3]
        for freq  in FreqArry:                                      #LOOP: Frequency
            NR5G.Freq   = freq
            NR5G.SMW.Set_Freq(freq)
            NR5G.FSW.Set_Freq(freq + CCStart)
            print(f'Freq:{freq:.0f} RFBW:{NR5G.NR_ChBW} SubC:{NR5G.NR_SubSp} Mod:{NR5G.NR_Mod}')
            print(Header)
            for CC in range(NumCC):
                NR5G.FSW.cc = CC+1
                NR5G.FSW.Set_5GNR_CC_Offset(CC+1,CC*CCSpace)
            for pwr in pwrArry:                                     #LOOP: Power
                tick = timeit.default_timer()                       #Tick Begin meas
                NR5G.SMW.Set_RFPwr(pwr)
                NR5G.FSW.Set_5GNR_CC_Capture('AUTO')
                NR5G.FSW.Set_SweepCont(1)
                NR5G.FSW.Set_Autolevel()
                NR5G.FSW.Set_5GNR_CC_Capture('SING')
                tockA = timeit.default_timer()                      #Tick Auto Lev
                NR5G.FSW.Set_SweepCont(0)
                NR5G.FSW.Set_InitImm()
                tockB    = timeit.default_timer()                   #Tick Meas
                ALTime   = (tockA-tick)
                TotTime  = (tockB-tick)
                TestTime = TotTime - ALTime

                ##############################################################
                ### LOG DATA
                ##############################################################
                for CC in range(NumCC):                             #LOOP: CC
                    NR5G.FSW.cc = CC+1
                    CurrFreq    = NR5G.FSW.Get_5GNR_CC_Freq()
                    LoopParam   = f'{i},{NR5G.FSW.Model},{freq},{CurrFreq/1e9:9.6f},{pwr:3d},{NR5G.FSW.cc},{NumCC}'
                    NR5GParam   = f'{NR5G.NR_ChBW},{NR5G.NR_RB},{NR5G.NR_SubSp},{NR5G.NR_Mod},{NR5G.NR_TF}'
                    AttnParam   = NR5G.FSW.Get_Params_Amp()
                    EVM         = NR5G.FSW.Get_5GNR_Params_EVM()
                    TimeParam   = f'{ALTime:2,.3f},{TestTime:2,.3f},{TotTime:2,.3f}'
                    OutStr      = f'{LoopParam},{NR5GParam},{AttnParam},{EVM},{TimeParam}'
                    OFile.write(OutStr)

###############################################################################
### Cleanup Automation
###############################################################################
NR5G.jav_Close()



"""
Terms:  Azimuth:    Θθ; Theta; Turntable
        Elevation:  Φφ; Phi; Cradle
OTA:    ATS000      Az over El
        ATS1500     Az over El
        ATS1800     Az over El
"""
from rssd.OTA.Common     import OTA           #pylint: disable=E0611,E0401

class OTA(OTA):
    """ Rohde & Schwarz ATS1800 Object """
    def __init__(self):
        super(OTA, self).__init__()
        self.Model = "ATS1800"
        self.EOL   = '\x00'

    #####################################################################
    ### OTA Get Functions
    #####################################################################
    def Get_AzimuthAngle(self):
        rdStr = self.query(f'SENS:AZIM:POS?')
        return rdStr

    def Get_AzimuthRunning(self):
        rdStr = self.query(f'SENS:AZIM:BUSY?')
        return rdStr

    def Get_AzimuthSpeed(self):
        rdStr = self.query(f'CONT:AZIM:SPE?')
        return rdStr

    def Get_ElevateAngle(self):
        rdStr = self.query(f'SENS:ELEV:POS?')
        return rdStr

    def Get_ElevateRunning(self):
        rdStr = self.query(f'SENS:ELEV:BUSY?')
        return rdStr

    def Get_ElevateSpeed(self):
        rdStr = self.query(f'CONT:ELEV:SPE?')
        return rdStr

    def Get_SysStat(self):
        rdStr = self.query('SYST:STAT?')
        return rdStr

    #####################################################################
    ### OTA Init Functions
    #####################################################################
    def Init_Measurement(self):
        #Configure instrument measurment
        pass

    #####################################################################
    ### OTA Set Functions
    #####################################################################
    def Set_AzimuthAngle(self,angle):
        self.query(f'CONT:AZIM:POS:TARG {angle}')
        self.query(f'CONT:AZIM:STAR')

    def Set_AzimuthSpeed(self,speed):
        """Speed: 1 to 150 degree/sec"""
        self.query(f'CONT:AZIM:SPE {speed}')

    def Set_AzimuthStop(self):
        self.query(f'CONT:AZIM:STOP')

    def Set_ElevateAngle(self,angle):
        self.query(f'CONT:ELEV:POS:TARG {angle}')
        self.query(f'CONT:ELEV:STAR')

    def Set_ElevateSpeed(self,speed):
        """Speed: 1 to 150 degree/sec"""
        self.query(f'CONT:ELEV:SPE {speed}')

    def Set_ElevateStop(self):
        self.query(f'CONT:ELEV:STOP')

###############################################################################
### Debug Main.  Won't run when imported
###############################################################################
if __name__ == "__main__":
    # http://169.254.2.10/Tc3PlcHmiWeb/Port_851/Visu/webvisu.htm
    ATS1800 = OTA().open('169.254.2.10', type = 'socket', param = 200)
    ATS1800.Set_ElevateAngle(40)
    print(ATS1800.Get_ElevateRunning())
    print(ATS1800.Get_ElevateAngle())



"""test rssd.yaVISA_socket
"""
from __future__ import print_function
#pylint: disable=import-outside-toplevel
import unittest

class TestGeneral(unittest.TestCase):
    def setUp(self):                                #Run before each test
        print("",end="")

    def tearDown(self):                             #Run after each test
        try:
            self.instr.jav_Close()
        except:
            pass

###############################################################################
### <Test>
###############################################################################
    def test_yaVISA_delay(self):
        from rssd.yaVISA import jaVisa
        self.instr = jaVisa()
        self.instr.delay(0.1)

    # def test_yaVISA_noVISA(self):
    #     # Test will not make a VISA connection.  Testing exception paths
    #     from rssd.yaVISA import jaVisa
    #     self.instr = jaVisa()
    #     # self.instr.K2.settimeout(0.01)
    #     self.instr.jav_Open('1.1.1.1')
    #     self.instr.jav_ClrErr                       # except
    #     self.assertTrue(1)

    def test_yaVISAs_delay(self):
        from rssd.yaVISA_socket import jaVisa
        self.instr = jaVisa()
        self.instr.delay(0.1)

    def test_yaVISAs_Open(self):
        setting = 1.23
        from rssd.yaVISA_socket import jaVisa       #pylint:disable=E0611,E0401
        self.Instr = jaVisa()
        self.Instr.debug = 0
        # self.Instr.jav_Open('www.yahoo.com',port=80)
        # self.Instr.K2.settimeout(setting)
        # rdStr = self.Instr.K2.gettimeout()
        # self.Instr.jav_Close()
        # self.assertEqual(setting,rdStr)

###############################################################################
### </Test>
###############################################################################
if __name__ == '__main__':
    # unittest.main()     #Run w/o test names
    suite = unittest.TestLoader().loadTestsFromTestCase(TestGeneral)
    unittest.TextTestRunner(verbosity=2).run(suite)



"""5G NR FSW/SMW Carrier Aggregation Setup Example"""
###############################################################################
### User Entry
###############################################################################
#pylint: disable=E0401
#pylint: disable=E0611
# SMW_IP  = '172.24.225.230'
SMW_IP  = '192.168.58.114'
FSW_IP  = '192.168.58.109'
VSG_ON  = 1
Freq    = 24e9
Pwr     = -5
NumCC   = 4
NR_Dir  = 'UL'
CCSpace = 99.96e6
modu    = 'QPSK'
CCStart = (1 - NumCC) * (CCSpace/2)

###############################################################################
### Code Overhead: Import and create objects
###############################################################################
import timeit
from rssd.VSG.NR5G_K144     import VSG
from rssd.VSA.NR5G_K144     import VSA
# from rssd.FileIO          import FileIO

if VSG_ON: SMW = VSG().jav_Open(SMW_IP)                 #Create SMW Object
FSW = VSA().jav_Open(FSW_IP)                            #Create FSW Object

###############################################################################
### Code Start
###############################################################################
if VSG_ON:
    SMW.Get_SysC_All()
    SMW.Set_Freq(Freq)                                  # SMW Freq --> BB Center
    SMW.Set_5GNR_BBState('OFF')
    SMW.Set_5GNR_Direction(NR_Dir)
    SMW.Set_5GNR_CC_Num(NumCC)
    SMW.Set_5GNR_PhaseCompensate('OFF')
    SMW.Set_RFPwr(Pwr)
    SMW.Set_RFState(1)

FSW.Init_5GNR()
FSW.Set_5GNR_Result_View('ALL')
FSW.Set_5GNR_EVMUnit('DB')
FSW.Set_5GNR_Direction(NR_Dir)
FSW.Set_5GNR_FreqRange(2)
FSW.Set_SweepCont(1)
FSW.Set_5GNR_CC_Num(NumCC)
FSW.Set_5GNR_CC_Capture('SING')
FSW.Set_Freq(Freq + CCStart)                            # FSW Freq --> 1st CC

for i in range(NumCC):
    Freq_CC = Freq + CCStart + (i * CCSpace)
    if VSG_ON:
        SMW.cc = i
        SMW.Set_5GNR_FreqRange(2)
        SMW.Set_5GNR_CC_Offset(CCStart + (i * CCSpace))
        SMW.Set_5GNR_TransPrecoding('ON')
        # SMW.Set_5GNR_PhaseCompensate_Freq(Freq_CC)
        SMW.Set_5GNR_BWP_Ch_Modulation(modu)
    FSW.cc = i+1
    FSW.Set_5GNR_CC_Offset(i+1,i*CCSpace)
    FSW.Set_5GNR_TransPrecoding('ON')
    FSW.Set_5GNR_PhaseCompensate('OFF')
    FSW.Set_5GNR_PhaseCompensate_Freq(Freq_CC)
    FSW.Set_5GNR_CellID(i)
    FSW.Set_5GNR_BWP_SubSpace(120)
    FSW.Set_5GNR_BWP_Ch_Modulation(modu)

tick    = timeit.default_timer()
if VSG_ON: SMW.Set_5GNR_BBState('ON')
tockA   = timeit.default_timer()
print(f'Total Time: {(tockA-tick):2,.6f} sec')
FSW.Set_Autolevel()

###############################################################################
### Close Nicely
###############################################################################
if VSG_ON: SMW.jav_Close()
